/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Data Source Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Data Source Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Data Source Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Data Source Key] (	@FactTemplateTempTableName	VARCHAR(200),
																@DataSourceKeyColumnName	VARCHAR( 100 ),
																@DataSourceIdColumnName		VARCHAR( 100 ),
																@Debug						CHAR(1)
														)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =	N'
		DECLARE @DataSourceKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownDataSourceKey<@UniqueId>			TINYINT = ( SELECT [Data Source Key] FROM [dim].[Dim Data Source] WITH (NOLOCK) WHERE [Data Source Internal Id] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Data Source Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @DataSourceKeyStepTimeStart<@UniqueId> = GETDATE();
		END

		UPDATE fact
		SET [<@DataSourceKeyColumnName>] = ISNULL( dim.[Data Source Key], @UnknownDataSourceKey<@UniqueId>)
		FROM <@FactTemplateTempTableName> AS fact
				LEFT OUTER JOIN [dim].[Dim Data Source] AS dim WITH (NOLOCK) ON fact.[<@DataSourceIdColumnName>] = dim.[Data Source Internal Id]

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Data Source Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @DataSourceKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
			'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @DataSourceKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@DataSourceKeyColumnName>',  @DataSourceKeyColumnName );
	SELECT @Statement = REPLACE( @Statement, '<@DataSourceIdColumnName>',  @DataSourceIdColumnName );
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',  @FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Data Source Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Data Source Key]'
END
GO
