/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Date Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Date Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Date Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Date Key] ( @FactTemplateTempTableName	VARCHAR(200),
														@Date						DATE,
														@DateKeyColumnName			VARCHAR( 100 ),
														@DateColumnName				VARCHAR( 100 ),
														@Debug						CHAR(1)
														)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =		N'
		DECLARE @DateKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownDateKey<@UniqueId>				INT = ( SELECT [Date Key] FROM [dim].[Dim Date] WITH (NOLOCK) WHERE [Date Desc] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Date Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @DateKeyStepTimeStart<@UniqueId> = GETDATE();
		END


		IF ''<@Date>'' != ''NULL''
			UPDATE fact
			SET [<@DateKeyColumnName>] = <@Date>
			FROM <@FactTemplateTempTableName> AS fact
		ELSE --
			UPDATE fact
			SET [<@DateKeyColumnName>] = ISNULL( dim.[Date Key], @UnknownDateKey<@UniqueId>)
			FROM <@FactTemplateTempTableName> AS fact
					LEFT OUTER JOIN [dim].[Dim Date] AS dim WITH (NOLOCK) ON <@JoinCondition>

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Date Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @DateKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
			'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @DateKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',  @FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Date>', RTRIM ( ISNULL( CONVERT( CHAR(8), @Date, 112), 'NULL' ) ) );
	SELECT @Statement = REPLACE( @Statement, '<@DateKeyColumnName>', @DateKeyColumnName ) ;
	SELECT @Statement = REPLACE( @Statement, '<@JoinCondition>', ISNULL( 'fact.[' + @DateColumnName + '] = dim.[Date]' , '1=0' ) );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Date Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Date Key]'
END
GO
