/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Field Position Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Field Position Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Field Position Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Field Position Key] (	@FactTemplateTempTableName		VARCHAR(200),
																	@FieldPositionKeyColumnName		VARCHAR( 100 ),
																	@DownColumnName					VARCHAR (200),
																	@YardLineColumnName				VARCHAR (200),
																	@RushDirectionColumnName		VARCHAR (200),
																	@YardLineFixedColumnName		VARCHAR (200),
																	@YardLineDirectionColumnName	VARCHAR (200),
																	@Debug							CHAR(1)
																)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =	N'
		DECLARE @FieldPositionKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownFieldPositionKey<@UniqueId>				SMALLINT = ( SELECT [Field Position Key] FROM [dim].[Dim Field Position] WITH (NOLOCK) WHERE [Down] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Field Position Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @FieldPositionKeyStepTimeStart<@UniqueId> = GETDATE();
		END

		UPDATE fact
		SET [<@FieldPositionKeyColumnName>] = ISNULL( dim.[Field Position Key], @UnknownFieldPositionKey<@UniqueId>)
		FROM <@FactTemplateTempTableName> AS fact
				LEFT OUTER JOIN [dim].[Dim Field Position] AS dim WITH (NOLOCK) ON	fact.[<@DownColumnName>]				= dim.[Down]					
																				AND fact.[<@YardLineColumnName>]			= dim.[Yard Line]				
																				AND fact.[<@RushDirectionColumnName>]		= dim.[Rush Direction]		
																				AND fact.[<@YardLineFixedColumnName>]		= dim.[Yard Line Fixed]		
																				AND fact.[<@YardLineDirectionColumnName>]	= dim.[Yard Line Direction]	

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Field Position Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @FieldPositionKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
		
		IF ( SELECT [Is Report Unknown Keys In Facts] FROM [dwetl].[Data Mart Properties] ) = 1
		BEGIN
			DECLARE @Message<@UniqueId>			NVARCHAR(4000);
			DECLARE @MessageList<@UniqueId>		VARCHAR(MAX)
			CREATE TABLE #UnknownKeys<@UniqueId>
			(
				[Logical Key] VARCHAR (1000),
			)
			
			-- This part will be unique for each key setting function
			DECLARE @KeyColumnName<@UniqueId> VARCHAR( 1000 ) = ''[<@FormationColumnName>], [<@FieldPositionColumnName>], [<@PassTypeColumnName>], [<@PenaltyTypeColumnName>]''
			
			-- Exclude Unknown keys caused by absence of input values - they will be reported as part of Global Data Verification
			INSERT #UnknownKeys<@UniqueId>
			SELECT DISTINCT 
							   ''[<@DownColumnName>] = ''				+ '''''''' + ISNULL( [<@DownColumnName>], ''NULL'' )				+ ''''''''
						+ '' AND [<@YardLineColumnName>] = ''			+ '''''''' + ISNULL( [<@YardLineColumnName>], ''NULL'' )			+ ''''''''
						+ '' AND [<@RushDirectionColumnName>] = ''		+ '''''''' + ISNULL( [<@RushDirectionColumnName>], ''NULL'' )		+ ''''''''
						+ '' AND [<@YardLineFixedColumnName>] = ''		+ '''''''' + ISNULL( [<@YardLineFixedColumnName>], ''NULL'' )		+ ''''''''
						+ '' AND [<@YardLineDirectionColumnName>] = ''	+ '''''''' + ISNULL( [<@YardLineDirectionColumnName>], ''NULL'' )	+ ''''''''
			FROM <@FactTemplateTempTableName> AS fact
			WHERE [<@FieldPositionKeyColumnName>] = @UnknownFieldPositionKey<@UniqueId>
				-- AND NULLIF( [<@FieldPositionIdColumnName>], '''' ) IS NOT NULL
				
			--This code is the same for all keysetting functions - can potentialty be moved into a stand alone function for code duplication reduction
			IF EXISTS ( SELECT 1 FROM #UnknownKeys<@UniqueId> )
			BEGIN
				DECLARE @CountUnknownKeys<@UniqueId> INT = ( SELECT COUNT(*) FROM #UnknownKeys<@UniqueId> )

				SET @Message<@UniqueId> = ''ETL WARNING!'' + CAST( @CountUnknownKeys<@UniqueId> AS VARCHAR(25)) + '' '' + 
									@KeyColumnName<@UniqueId> + '' values are UNKNOWN! The following logical keys are set to Unknown. Please check. '' + CHAR(13) + CHAR(10)
				SET @Message<@UniqueId> += ''The first 10 logical keys with the issue '' + CHAR(13) + CHAR(10)
				
				SET @MessageList<@UniqueId> = ''''
				SELECT @MessageList<@UniqueId> = COALESCE( @MessageList<@UniqueId> + CHAR(13) + CHAR(10), '''' ) + ISNULL( [Logical Key] + '''', ''NULL'' )
				FROM (	SELECT TOP ( 10 ) [Logical Key]
						FROM #UnknownKeys<@UniqueId> ) AS unknown 
				SET @Message<@UniqueId> = @Message<@UniqueId> + @MessageList<@UniqueId> + CHAR(13) + CHAR(10)
				PRINT CAST ( @Message<@UniqueId> AS TEXT );
			END
		END		
		'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @FieldPositionKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@FieldPositionKeyColumnName>',	@FieldPositionKeyColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@DownColumnName>',				@DownColumnName					);
	SELECT @Statement = REPLACE( @Statement, '<@YardLineColumnName>',			@YardLineColumnName				);
	SELECT @Statement = REPLACE( @Statement, '<@RushDirectionColumnName>',		@RushDirectionColumnName		);
	SELECT @Statement = REPLACE( @Statement, '<@YardLineFixedColumnName>',		@YardLineFixedColumnName		);
	SELECT @Statement = REPLACE( @Statement, '<@YardLineDirectionColumnName>',	@YardLineDirectionColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',  @FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Field Position Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Field Position Key]'
END
GO
