/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Play Details Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Play Details Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Play Details Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Play Details Key] (	@FactTemplateTempTableName					VARCHAR(200),
																@PlayDetailsKeyColumnName					VARCHAR( 100 ),
																@IsRushColumnName							VARCHAR (200),
																@IsPassColumnName							VARCHAR (200),
																@IsIncompleteColumnName						VARCHAR (200),
																@IsTouchdownColumnName						VARCHAR (200),
																@IsSackColumnName							VARCHAR (200),
																@IsChallengeColumnName						VARCHAR (200),
																@IsChallengeReversedColumnName				VARCHAR (200),
																@IsMeasurementColumnName					VARCHAR (200),
																@IsInterceptionColumnName					VARCHAR (200),
																@IsFumbleColumnName							VARCHAR (200),
																@IsPenaltyColumnName						VARCHAR (200),
																@IsTwoPointConversionColumnName				VARCHAR (200),
																@IsTwoPointConversionSuccessfulColumnName	VARCHAR (200),
																@IsPenaltyAcceptedColumnName				VARCHAR (200),
																@IsNoPlayColumnName							VARCHAR (200),
																@Debug										CHAR(1)
															)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =	N'
		DECLARE @PlayDetailsKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownPlayDetailsKey<@UniqueId>			SMALLINT = ( SELECT [Play Details Key] FROM [dim].[Dim Play Details] WITH (NOLOCK) WHERE [Is Rush] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Play Details Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @PlayDetailsKeyStepTimeStart<@UniqueId> = GETDATE();
		END

		UPDATE fact
		SET [<@PlayDetailsKeyColumnName>] = ISNULL( dim.[Play Details Key], @UnknownPlayDetailsKey<@UniqueId>)
		FROM <@FactTemplateTempTableName> AS fact
				LEFT OUTER JOIN [dim].[Dim Play Details] AS dim WITH (NOLOCK) ON	fact.[<@IsRushColumnName>]							= dim.[Is Rush]
																				AND fact.[<@IsPassColumnName>]							= dim.[Is Pass]
																				AND fact.[<@IsIncompleteColumnName>]					= dim.[Is Incomplete]
																				AND fact.[<@IsTouchdownColumnName>]						= dim.[Is Touchdown]
																				AND fact.[<@IsSackColumnName>]							= dim.[Is Sack]
																				AND fact.[<@IsChallengeColumnName>]						= dim.[Is Challenge]
																				AND fact.[<@IsChallengeReversedColumnName>]				= dim.[Is Challenge Reversed]
																				AND fact.[<@IsMeasurementColumnName>]					= dim.[Is Measurement]
																				AND fact.[<@IsInterceptionColumnName>]					= dim.[Is Interception]
																				AND fact.[<@IsFumbleColumnName>]						= dim.[Is Fumble]
																				AND fact.[<@IsPenaltyColumnName>]						= dim.[Is Penalty]
																				AND fact.[<@IsTwoPointConversionColumnName>]			= dim.[Is Two Point Conversion]
																				AND fact.[<@IsTwoPointConversionSuccessfulColumnName>]	= dim.[Is Two Point Conversion Successful]
																				AND fact.[<@IsPenaltyAcceptedColumnName>]				= dim.[Is Penalty Accepted]
																				AND fact.[<@IsNoPlayColumnName>]						= dim.[Is No Play]

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Play Details Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @PlayDetailsKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
		
		IF ( SELECT [Is Report Unknown Keys In Facts] FROM [dwetl].[Data Mart Properties] ) = 1
		BEGIN
			DECLARE @Message<@UniqueId>			NVARCHAR(4000);
			DECLARE @MessageList<@UniqueId>		VARCHAR(MAX)
			CREATE TABLE #UnknownKeys<@UniqueId>
			(
				[Logical Key] VARCHAR (1000),
			)
			
			-- This part will be unique for each key setting function
			DECLARE @KeyColumnName<@UniqueId> VARCHAR( 1000 ) = ''[<@IsRushColumnName>], [<@IsPassColumnName>], [<@IsIncompleteColumnName>], [<@IsTouchdownColumnName>], [<@IsSackColumnName>], [<@IsChallengeColumnName>], [<@IsChallengeReversedColumnName>], [<@IsMeasurementColumnName>],	[<@IsInterceptionColumnName>], [<@IsFumbleColumnName>], [<@IsPenaltyColumnName>], [<@IsTwoPointConversionColumnName>], [<@IsTwoPointConversionSuccessfulColumnName>], [<@IsPenaltyAcceptedColumnName>],	[<@IsNoPlayColumnName>]''
			
			-- Exclude Unknown keys caused by absence of input values - they will be reported as part of Global Data Verification
			INSERT #UnknownKeys<@UniqueId>
			SELECT DISTINCT 
							   ''[<@IsRushColumnName>]							 = ''		+ '''''''' + ISNULL( [<@IsRushColumnName>]							, ''NULL'' ) + ''''''''
						+ '' AND [<@IsPassColumnName>]							 = ''		+ '''''''' + ISNULL( [<@IsPassColumnName>]							, ''NULL'' ) + ''''''''
						+ '' AND [<@IsIncompleteColumnName>]					 = ''		+ '''''''' + ISNULL( [<@IsIncompleteColumnName>]					, ''NULL'' ) + ''''''''
						+ '' AND [<@IsTouchdownColumnName>]						 = ''		+ '''''''' + ISNULL( [<@IsTouchdownColumnName>]						, ''NULL'' ) + ''''''''
						+ '' AND [<@IsSackColumnName>]							 = ''		+ '''''''' + ISNULL( [<@IsSackColumnName>]							, ''NULL'' ) + ''''''''
						+ '' AND [<@IsChallengeColumnName>]						 = ''		+ '''''''' + ISNULL( [<@IsChallengeColumnName>]						, ''NULL'' ) + ''''''''
						+ '' AND [<@IsChallengeReversedColumnName>]				 = ''		+ '''''''' + ISNULL( [<@IsChallengeReversedColumnName>]				, ''NULL'' ) + ''''''''
						+ '' AND [<@IsMeasurementColumnName>]					 = ''		+ '''''''' + ISNULL( [<@IsMeasurementColumnName>]					, ''NULL'' ) + ''''''''
						+ '' AND [<@IsInterceptionColumnName>]					 = ''		+ '''''''' + ISNULL( [<@IsInterceptionColumnName>]					, ''NULL'' ) + ''''''''
						+ '' AND [<@IsFumbleColumnName>]						 = ''		+ '''''''' + ISNULL( [<@IsFumbleColumnName>]						, ''NULL'' ) + ''''''''
						+ '' AND [<@IsPenaltyColumnName>]						 = ''		+ '''''''' + ISNULL( [<@IsPenaltyColumnName>]						, ''NULL'' ) + ''''''''
						+ '' AND [<@IsTwoPointConversionColumnName>]			 = ''		+ '''''''' + ISNULL( [<@IsTwoPointConversionColumnName>]			, ''NULL'' ) + ''''''''
						+ '' AND [<@IsTwoPointConversionSuccessfulColumnName>]	 = ''		+ '''''''' + ISNULL( [<@IsTwoPointConversionSuccessfulColumnName>]	, ''NULL'' ) + ''''''''
						+ '' AND [<@IsPenaltyAcceptedColumnName>]				 = ''		+ '''''''' + ISNULL( [<@IsPenaltyAcceptedColumnName>]				, ''NULL'' ) + ''''''''
						+ '' AND [<@IsNoPlayColumnName>]						 = ''		+ '''''''' + ISNULL( [<@IsNoPlayColumnName>]						, ''NULL'' ) + ''''''''
			FROM <@FactTemplateTempTableName> AS fact
			WHERE [<@PlayDetailsKeyColumnName>] = @UnknownPlayDetailsKey<@UniqueId>
				
			--This code is the same for all keysetting functions - can potentialty be moved into a stand alone function for code duplication reduction
			IF EXISTS ( SELECT 1 FROM #UnknownKeys<@UniqueId> )
			BEGIN
				DECLARE @CountUnknownKeys<@UniqueId> INT = ( SELECT COUNT(*) FROM #UnknownKeys<@UniqueId> )

				SET @Message<@UniqueId> = ''ETL WARNING!'' + CAST( @CountUnknownKeys<@UniqueId> AS VARCHAR(25)) + '' '' + 
									@KeyColumnName<@UniqueId> + '' values are UNKNOWN! The following logical keys are set to Unknown. Please check. '' + CHAR(13) + CHAR(10)
				SET @Message<@UniqueId> += ''The first 10 logical keys with the issue '' + CHAR(13) + CHAR(10)
				
				SET @MessageList<@UniqueId> = ''''
				SELECT @MessageList<@UniqueId> = COALESCE( @MessageList<@UniqueId> + CHAR(13) + CHAR(10), '''' ) + ISNULL( [Logical Key] + '''', ''NULL'' )
				FROM (	SELECT TOP ( 10 ) [Logical Key]
						FROM #UnknownKeys<@UniqueId> ) AS unknown 
				SET @Message<@UniqueId> = @Message<@UniqueId> + @MessageList<@UniqueId> + CHAR(13) + CHAR(10)
				PRINT CAST ( @Message<@UniqueId> AS TEXT );
			END
		END		
		'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @PlayDetailsKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@PlayDetailsKeyColumnName>',					@PlayDetailsKeyColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsRushColumnName>',							@IsRushColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsPassColumnName>',							@IsPassColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsIncompleteColumnName>',					@IsIncompleteColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsTouchdownColumnName>',					@IsTouchdownColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsSackColumnName>',							@IsSackColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsChallengeColumnName>',					@IsChallengeColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsChallengeReversedColumnName>',			@IsChallengeReversedColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsMeasurementColumnName>',					@IsMeasurementColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsInterceptionColumnName>',					@IsInterceptionColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsFumbleColumnName>',						@IsFumbleColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsPenaltyColumnName>',						@IsPenaltyColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsTwoPointConversionColumnName>',			@IsTwoPointConversionColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsTwoPointConversionSuccessfulColumnName>',	@IsTwoPointConversionSuccessfulColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsPenaltyAcceptedColumnName>',				@IsPenaltyAcceptedColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@IsNoPlayColumnName>',						@IsNoPlayColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',	@FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Play Details Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Play Details Key]'
END
GO
