/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Play Time Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Play Time Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Play Time Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Play Time Key] (	@FactTemplateTempTableName	VARCHAR(200),
																@PlayTimeKeyColumnName		VARCHAR( 100 ),
																@QuarterColumnName			VARCHAR (200),
																@MinuteColumnName			VARCHAR (200),
																@SecondColumnName			VARCHAR (200),
																@Debug						CHAR(1)
															)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =	N'
		DECLARE @PlayTimeKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownPlayTimeKey<@UniqueId>				INT = ( SELECT [Play Time Key] FROM [dim].[Dim Play Time] WITH (NOLOCK) WHERE [Quarter] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Play Time Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @PlayTimeKeyStepTimeStart<@UniqueId> = GETDATE();
		END

		UPDATE fact
		SET [<@PlayTimeKeyColumnName>] = ISNULL( dim.[Play Time Key], @UnknownPlayTimeKey<@UniqueId>)
		FROM <@FactTemplateTempTableName> AS fact
				LEFT OUTER JOIN [dim].[Dim Play Time] AS dim WITH (NOLOCK) ON		fact.[<@QuarterColumnName>]	= dim.[Quarter]
																				AND fact.[<@MinuteColumnName>]	= dim.[Minute]
																				AND fact.[<@SecondColumnName>]	= dim.[Second]

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Play Time Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @PlayTimeKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
		
		IF ( SELECT [Is Report Unknown Keys In Facts] FROM [dwetl].[Data Mart Properties] ) = 1
		BEGIN
			DECLARE @Message<@UniqueId>			NVARCHAR(4000);
			DECLARE @MessageList<@UniqueId>		VARCHAR(MAX)
			CREATE TABLE #UnknownKeys<@UniqueId>
			(
				[Logical Key] VARCHAR (1000),
			)
			
			-- This part will be unique for each key setting function
			DECLARE @KeyColumnName<@UniqueId> VARCHAR( 1000 ) = ''[<@QuarterColumnName>], [<@MinuteColumnName>], [<@SecondColumnName>]''
			
			-- Exclude Unknown keys caused by absence of input values - they will be reported as part of Global Data Verification
			INSERT #UnknownKeys<@UniqueId>
			SELECT DISTINCT 
							''[<@QuarterColumnName>] = ''			+ '''''''' + ISNULL( [<@QuarterColumnName>], ''NULL'' ) + ''''''''
						+ '' AND [<@MinuteColumnName>] = ''		+ '''''''' + ISNULL( [<@MinuteColumnName>], ''NULL'' ) + ''''''''
						+ '' AND [<@SecondColumnName>] = ''		+ '''''''' + ISNULL( [<@SecondColumnName>], ''NULL'' ) + ''''''''
			FROM <@FactTemplateTempTableName> AS fact
			WHERE [<@PlayTimeKeyColumnName>] = @UnknownPlayTimeKey<@UniqueId>
				
			--This code is the same for all keysetting functions - can potentialty be moved into a stand alone function for code duplication reduction
			IF EXISTS ( SELECT 1 FROM #UnknownKeys<@UniqueId> )
			BEGIN
				DECLARE @CountUnknownKeys<@UniqueId> INT = ( SELECT COUNT(*) FROM #UnknownKeys<@UniqueId> )

				SET @Message<@UniqueId> = ''ETL WARNING!'' + CAST( @CountUnknownKeys<@UniqueId> AS VARCHAR(25)) + '' '' + 
									@KeyColumnName<@UniqueId> + '' values are UNKNOWN! The following logical keys are set to Unknown. Please check. '' + CHAR(13) + CHAR(10)
				SET @Message<@UniqueId> += ''The first 10 logical keys with the issue '' + CHAR(13) + CHAR(10)
				
				SET @MessageList<@UniqueId> = ''''
				SELECT @MessageList<@UniqueId> = COALESCE( @MessageList<@UniqueId> + CHAR(13) + CHAR(10), '''' ) + ISNULL( [Logical Key] + '''', ''NULL'' )
				FROM (	SELECT TOP ( 10 ) [Logical Key]
						FROM #UnknownKeys<@UniqueId> ) AS unknown 
				SET @Message<@UniqueId> = @Message<@UniqueId> + @MessageList<@UniqueId> + CHAR(13) + CHAR(10)
				PRINT CAST ( @Message<@UniqueId> AS TEXT );
			END
		END		
		'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @PlayTimeKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@PlayTimeKeyColumnName>',	@PlayTimeKeyColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@QuarterColumnName>',		@QuarterColumnName	);
	SELECT @Statement = REPLACE( @Statement, '<@MinuteColumnName>',			@MinuteColumnName		);
	SELECT @Statement = REPLACE( @Statement, '<@SecondColumnName>',			@SecondColumnName		);
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',  @FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Play Time Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Play Time Key]'
END
GO
