/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Team Key]'))
BEGIN
	DROP FUNCTION [dim].[Get Statement To Set Team Key]
	PRINT 'DROPED FUNCTION [dim].[Get Statement To Set Team Key]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE FUNCTION [dim].[Get Statement To Set Team Key] (	@FactTemplateTempTableName	VARCHAR(200),
														@EffectiveDate				DATE,
														@EffectiveDateColumnName	VARCHAR( 100 ),
														@TeamKeyColumnName			VARCHAR( 100 ),
														@TeamAliasColumnName		VARCHAR( 100 ),
														@Debug						CHAR(1)
																											)
RETURNS NVARCHAR(MAX)
AS
BEGIN
	DECLARE @Statement				NVARCHAR(MAX)
	
	SELECT @Statement =	N'
		DECLARE @TeamKeyStepTimeStart<@UniqueId>		DATETIME
		DECLARE @UnknownTeamKey<@UniqueId>				INT = ( SELECT [Team Key] FROM [dim].[Dim Team] WITH (NOLOCK) WHERE [Team Original Name] = ''Unknown'' )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point before Team Key. ''  + RTRIM( CONVERT(CHAR(40), GETDATE(), 8));
			SET @TeamKeyStepTimeStart<@UniqueId> = GETDATE();
		END

		UPDATE fact
		SET [<@TeamKeyColumnName>] = ISNULL( (	SELECT [Team Key] 
												FROM [dim].[Get Team Key] 
														(	CASE WHEN ''<@EffectiveDate>'' != ''NULL'' 
																 THEN ''<@EffectiveDate>'' 
																 ELSE fact.[<@EffectiveDateColumnName>] 
																 END,										-- @EffectiveDate						
															NULL,											-- @TeamOriginalName ( NULL for now	)
															NULLIF( fact.[<@TeamAliasColumnName>], '''' )	-- @TeamAlias						
														)
											  ), @UnknownTeamKey<@UniqueId>
											)
		FROM <@FactTemplateTempTableName> AS fact

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN
			PRINT ''Point after setting Team Key. '' + CONVERT( VARCHAR(6),DATEDIFF(ss, @TeamKeyStepTimeStart<@UniqueId>, GETDATE())) + '' sec '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		END
		
		IF ( SELECT [Is Report Unknown Keys In Facts] FROM [dwetl].[Data Mart Properties] ) = 1
		BEGIN
			DECLARE @Message<@UniqueId>			NVARCHAR(4000);
			DECLARE @MessageList<@UniqueId>		VARCHAR(MAX)
			CREATE TABLE #UnknownKeys<@UniqueId>
			(
				[Logical Key] VARCHAR (1000),
			)
			
			-- This part will be unique for each key setting function
			DECLARE @KeyColumnName<@UniqueId> VARCHAR( 100 ) = ''[<@TeamAliasColumnName>]''
			
			-- Exclude Unknown keys caused by absence of input values - they will be reported as part of Global Data Verification
			INSERT #UnknownKeys<@UniqueId>
			SELECT DISTINCT ''[<@TeamAliasColumnName>] = '' + '''''''' + ISNULL( [<@TeamAliasColumnName>], ''NULL'' ) + ''''''''
			FROM <@FactTemplateTempTableName> AS fact
			WHERE [<@TeamKeyColumnName>] = @UnknownTeamKey<@UniqueId>
				AND NULLIF( [<@TeamAliasColumnName>], '''' ) IS NOT NULL
				
			--This code is the same for all keysetting functions - can potentialty be moved into a stand alone function for code duplication reduction
			IF EXISTS ( SELECT 1 FROM #UnknownKeys<@UniqueId> )
			BEGIN
				DECLARE @CountUnknownKeys<@UniqueId> INT = ( SELECT COUNT(*) FROM #UnknownKeys<@UniqueId> )

				SET @Message<@UniqueId> = ''ETL WARNING!'' + CAST( @CountUnknownKeys<@UniqueId> AS VARCHAR(25)) + '' '' + 
									@KeyColumnName<@UniqueId> + '' values are UNKNOWN! The following logical keys are set to Unknown. Please check. '' + CHAR(13) + CHAR(10)
				SET @Message<@UniqueId> += ''The first 10 logical keys with the issue '' + CHAR(13) + CHAR(10)
				
				SET @MessageList<@UniqueId> = ''''
				SELECT @MessageList<@UniqueId> = COALESCE( @MessageList<@UniqueId> + CHAR(13) + CHAR(10), '''' ) + ISNULL( [Logical Key] + '''', ''NULL'' )
				FROM (	SELECT TOP ( 10 ) [Logical Key]
						FROM #UnknownKeys<@UniqueId> ) AS unknown 
				SET @Message<@UniqueId> = @Message<@UniqueId> + @MessageList<@UniqueId> + CHAR(13) + CHAR(10)
				PRINT CAST ( @Message<@UniqueId> AS TEXT );
			END
		END		
		'

	SELECT @Statement = REPLACE( @Statement, '<@UniqueId>', CONVERT(VARCHAR(255), ( SELECT HASHBYTES( 'MD5', @TeamKeyColumnName ) ), 2 ) );
	SELECT @Statement = REPLACE( @Statement, '<@EffectiveDate>', RTRIM ( ISNULL( CONVERT( CHAR(8), @EffectiveDate, 112), 'NULL' ) ) );
	SELECT @Statement = REPLACE( @Statement, '<@EffectiveDateColumnName>', @EffectiveDateColumnName ) ;
	SELECT @Statement = REPLACE( @Statement, '<@TeamKeyColumnName>',  @TeamKeyColumnName );
	SELECT @Statement = REPLACE( @Statement, '<@TeamAliasColumnName>',  @TeamAliasColumnName );
	SELECT @Statement = REPLACE( @Statement, '<@FactTemplateTempTableName>',  @FactTemplateTempTableName );
	SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug );
	
	RETURN @Statement
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dim].[Get Statement To Set Team Key]'))
BEGIN
	PRINT 'CREATED FUNCTION [dim].[Get Statement To Set Team Key]'
END
GO
