/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO


DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [ssas].[*] and [ssasprocess].[*] objects' -- the objects include stored procedures, views, synonyms. When views are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('ssasdim' )
		AND ( obj.[type] = 'V'  ) 
ORDER BY obj.[type]
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'V' THEN 'VIEW'
												WHEN 'SN' THEN 'SYNONYM'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart%'

-- Create schema
DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ( 'ssasdim' )

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

CREATE VIEW [ssasdim].[Data Source]
AS
SELECT
	[Data Source Key]			,
	[Data Source Internal Id]	,
	[Data Source Name]			,
--	[Data Source Description]	,
	[Primary Record Name]		,
	[Transaction Name]			
FROM  [dim].[Dim Data Source]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Data Source]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Data Source]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Data Source]'
END
GO

CREATE VIEW [ssasdim].[Date]
AS 
SELECT
	[Date Key],
	[Date] 					,
	[Date Desc] 			,
	[Month Name]			,
	[Year]					,
	[Quarter]				,
	[Quarter Desc]			,
	YEAR( [Season Year]	)														AS 	[Season Year],
	[Season Description]	,
	CASE [Date Desc] WHEN 'Unknown' THEN 0 
			ELSE CAST( LEFT( CONVERT(VARCHAR(8), [Date], 112), 6 ) AS INT) END 	AS [Z Month Name Sort]
FROM [dim].[Dim Date]
WHERE [Date Desc] != 'Unknown' -- We only load specific dates
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Date]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Date]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Date]'
END
GO

CREATE VIEW [ssasdim].[Field Position]
AS 
SELECT
	[Field Position Key]		,	
	[Down]						,	
	[Yard Line]					,	
	[Rush Direction]			,	
	[Yard Line Fixed]			,	
	[Yard Line Direction]			
FROM [dim].[Dim Field Position]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Field Position]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Field Position]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Field Position]'
END
GO

CREATE VIEW [ssasdim].[Game]
AS 
SELECT
	[Game Key]		,	
	[Game Id]		,	
	[Game Teams]
FROM [dim].[Dim Game]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Game]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Game]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Game]'
END
GO

CREATE VIEW [ssasdim].[Play Details]
AS 
SELECT
	[Play Details Key]						,	
	[Is Rush]								,	
	[Is Pass]								,	
	[Is Incomplete]							,	
	[Is Touchdown]							,	
	[Is Sack]								,	
	[Is Challenge]							,	
	[Is Challenge Reversed]					,	
	[Is Measurement]						,	
	[Is Interception]						,	
	[Is Fumble]								,	
	[Is Penalty]							,	
	[Is Two Point Conversion]				,	
	[Is Two Point Conversion Successful]	,	
	[Is Penalty Accepted]					,	
	[Is No Play]							
FROM [dim].[Dim Play Details]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Play Details]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Play Details]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Play Details]'
END
GO

CREATE VIEW [ssasdim].[Play Time]
AS 
SELECT
	[Play Time Key]	,	
	[Quarter]		,	
	[Minute]		,	
	[Second]
FROM [dim].[Dim Play Time]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Play Time]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Play Time]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Play Time]'
END
GO

CREATE VIEW [ssasdim].[Play Type]
AS 
SELECT
	[Play Type Key]			,	
	[Formation]				,	
	[Play Type]				,	
	[Pass Type]				,	
	[Penalty Type]
FROM [dim].[Dim Play Type]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Play Type]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Play Type]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Play Type]'
END
GO

-- SELECT * FROM [ssasdim].[Team]
CREATE VIEW [ssasdim].[Team]
AS 
SELECT
	[Team Key]				,	
	[Team Original Name]	,	
	[Team Name]				,	
	[Team Alias]			,	
	[Team Alias]			AS [Offense Team]	,	
	[Team Alias]			AS [Defense Team]	,	
	[Team Alias]			AS [Penalty Team]	,	
	[Team Alias]			AS [Timeout Team]	,	
	[Team City]				,	
	[Team Conference]		,	
	[Sport Abbrev]			,	
	[Hist Team Name]	
FROM [dim].[Dim Team]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Team]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Team]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Team]'
END
GO

CREATE VIEW [ssasdim].[Team Role]
AS 
SELECT
	[Team Role Key]				,	
	[Team Role]	
FROM [dim].[Dim Team Role]
GO
IF ( SELECT OBJECT_ID( '[ssasdim].[Team Role]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasdim].[Team Role]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasdim].[Team Role]'
END
GO


SET ANSI_PADDING OFF

PRINT 'Created common SSAS views in ' + DB_NAME() + ' database'
