/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO


DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [ssas].[*] and [ssasprocess].[*] objects' -- the objects include stored procedures, views, synonyms. When views are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('ssasdimdm', 'ssasfact', 'ssasprocess')
		AND ( obj.[type] = 'V'  ) 
ORDER BY obj.[type]
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'V' THEN 'VIEW'
												WHEN 'SN' THEN 'SYNONYM'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart%'

-- Create schema
DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ( 'ssasdimdm' ), ( 'ssasfact' ), ( 'ssasprocess') -- the first one is dimensions spesific to Data Mart

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO
---------------------------------------------
-- Dimension views specific to this Data Mart
---------------------------------------------
-- ...


--------------
-- Fact views
--------------

-- Main Fact views
-- For each partitioned table create a view that will be used in 
-- SSAS cube definition and  generation

CREATE VIEW [ssasfact].[Plays]
AS
SELECT
	[Play Date Key]			,
	[Data Source Key]		,
	[Game Key]				,
	[Offense Team Key]		,
	[Defense Team Key]		,
	[Penalty Team Key]		,
	[Timeout Team Key]		,
	[Play Type Key]			,
	[Field Position Key]	,
	[Play Time Key]			,
	[Play Details Key]		,
	[Score]					,
	[Yards]					,
	[ToGo]					,
	[Penalty Yards]		
FROM [fact].[Fact Plays] WITH (NOLOCK) 
GO
IF ( SELECT OBJECT_ID( '[ssasfact].[Plays]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasfact].[Plays]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasfact].[Plays]'
END
GO

-- For each partitioned table create a view that will be used in 
-- SSAS cube definition and  generation
CREATE VIEW [ssasfact].[Plays partition]
AS
SELECT *
FROM [ssasfact].[Plays]
GO

CREATE VIEW [ssasfact].[Bridge Teams-Plays]
AS
SELECT
	[Data Source Key]			,
	[Team Key]					,
	[Team Role Key]				,
	[Game Key]					,
	[Play Type Key]				,
	[Field Position Key]		,
	[Play Time Key]			
FROM [fact].[Bridge Teams-Plays] WITH (NOLOCK) 
GO
IF ( SELECT OBJECT_ID( '[ssasfact].[Bridge Teams-Plays]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED VIEW  [ssasfact].[Bridge Teams-Plays]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING VIEW [ssasfact].[Bridge Teams-Plays]'
END
GO

-- For each partitioned table create a view that will be used in 
-- SSAS cube definition and  generation
CREATE VIEW [ssasfact].[Bridge Teams-Plays partition]
AS
SELECT *
FROM [ssasfact].[Bridge Teams-Plays]
GO

PRINT 'Created SSAS Data Mart views in ' + DB_NAME() + ' database'
