/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dimauth].[Load Static Dimensions]') AND type in (N'P', N'PC'))
BEGIN
	DROP PROCEDURE [dimauth].[Load Static Dimensions]
	PRINT 'DROPED PROCEDURE [dimauth].[Load Static Dimensions]'
END
GO
-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE PROCEDURE [dimauth].[Load Static Dimensions] ( 
									@Debug			CHAR(1) = 'N' 
)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON

	DECLARE @Error			INT
	DECLARE @RowsAffected	INT
	DECLARE @StepTimeStart	DATETIME
	DECLARE @TimeStart		DATETIME 

	DECLARE @Start		SMALLINT
	DECLARE @End		SMALLINT
	
	DECLARE @KeyTinyint		TINYINT
	DECLARE @KeySmallint	SMALLINT
	DECLARE @KeyInt			INT
	DECLARE @BandIncrement	INT

	DECLARE @TotalRowsInserted	INT = 0

	DECLARE @NEW_LINE_CHAR	CHAR(2) = CHAR(13) + CHAR(10)
	DECLARE @MAX_NUMBER_OF_AGE_MONTHS	SMALLINT = 480

	IF ( @Debug = 'Y' )
	BEGIN 
		PRINT 'Point 1. Start      ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		SET @TimeStart = GETDATE()
		SET @StepTimeStart = GETDATE()
	END

	BEGIN TRY

		PRINT '*** LOADING static dimensions'
		
		SET @TimeStart = GETDATE()

		-- Dim Data Source
		IF ( SELECT COUNT(*) FROM ( SELECT TOP 1 1 AS c FROM [dimauth].[Dim Data Source] ) AS t ) = 0 -- ## Populate [dimauth].[Dim Data Source]
		BEGIN
			INSERT [dimauth].[Dim Data Source] VALUES	( 'Unknown',			'Unknown',				'Unknown', '', '', DEFAULT, DEFAULT )				,
														( 'NFLSAVANT.COM',		'NFLSavant.com Play-By-Play data',	'[nflsavant].[...]', '', '', DEFAULT, DEFAULT )	
														 
			SELECT	@RowsAffected = @@ROWCOUNT
			PRINT 'Inserted ' + CAST( @RowsAffected AS VARCHAR(20)) + ' new rows in [dimauth].[Dim Data Source]'
		END

		-- Dim Team Role
		IF ( SELECT COUNT(*) FROM ( SELECT TOP 1 1 AS c FROM [dimauth].[Dim Team Role] ) AS t ) = 0 -- ## Populate [dimauth].[Dim Team Role]
		BEGIN
			INSERT [dimauth].[Dim Team Role] VALUES	( 'Unknown',		DEFAULT, DEFAULT )	,
													( 'Offense',		DEFAULT, DEFAULT )	,
													( 'Defense',		DEFAULT, DEFAULT )	,
													( 'Penalty',		DEFAULT, DEFAULT )	,
													( 'Timeout',		DEFAULT, DEFAULT )
														 
			SELECT	@RowsAffected = @@ROWCOUNT
			PRINT 'Inserted ' + CAST( @RowsAffected AS VARCHAR(20)) + ' new rows in [dimauth].[Dim Team Role]'
		END


		IF ( @Debug = 'Y' )
		BEGIN 
			PRINT 'Point last. ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
			PRINT '     Total time: ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		END

		PRINT '*** COMPLETED load of static dimensions in ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + ' sec at ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8)) + @NEW_LINE_CHAR

		RETURN 0
		
	END TRY
	BEGIN CATCH
	    DECLARE @ErrorMessage NVARCHAR(4000);
		DECLARE @ErrorSeverity INT;
		DECLARE @ErrorState INT;

		SELECT	@ErrorMessage  =
					  '   ERROR_NUMBER: ' + ISNULL( CONVERT(VARCHAR(6),ERROR_NUMBER()),'')  + CHAR(13)
					+ '   ERROR_SEVERITY: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_SEVERITY()),'')  + CHAR(13)
					+ '   ERROR_STATE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_STATE()),'')  + CHAR(13)
					+ '   ERROR_PROCEDURE: '  + ISNULL( ERROR_PROCEDURE(),'')  + CHAR(13)
					+ '   ERROR_LINE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_LINE()),'') + CHAR(13)
					+ '   ERROR_MESSAGE: '  + ISNULL( ERROR_MESSAGE(),'')  + CHAR(13),
				@ErrorSeverity = ERROR_SEVERITY(),
				@ErrorState = ERROR_STATE();
	
		IF @@TRANCOUNT > 0
			ROLLBACK TRANSACTION

	    RAISERROR ( @ErrorMessage, @ErrorSeverity, @ErrorState );

		RETURN 1
	END CATCH

END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dimauth].[Load Static Dimensions]') AND type IN (N'P', N'PC'))
BEGIN
	PRINT 'CREATED PROCEDURE [dimauth].[Load Static Dimensions]'
END
GO

