/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script]') AND type = 'P')
BEGIN
	DROP PROCEDURE [ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script]
	PRINT 'DROPED PROCEDURE [ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script]'
END
GO

BEGIN TRY
CREATE TABLE #ScriptCreateCube
(
	LineNumber	INT,
	LineText	VARCHAR(2000)
)
CREATE UNIQUE CLUSTERED INDEX ScriptCreateCube_Indx ON #ScriptCreateCube(LineNumber)
END TRY
BEGIN CATCH
END CATCH
GO

-- =============================================
-- Author:		Victor Shulman
-- Create date: 05-25-2013
-- Description:	The proc generates schema for a dimension or partition by replacing project DbTableName with the new one.
--              The proc expects the existence of #ScriptCreateCube table with project generated SSAS database definition
-- Parameters:  @ProjectMspropDbTableName - default table name from the DW project. WIll be used to identify the 
--											beginning of the schema definition in cube create script under assumption that the 
--											this name is unique, that the name is encountered only once in the schema and is 
--											located on the first line of object schema.
--				@NewMspropDbSchemaName	  - database schema for new view
--				@NewMspropDbTableName	  - database name of new view
--
-- =============================================
CREATE PROCEDURE [ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script] ( 
													@ProjectMspropDbTableName	VARCHAR(200),
													@NewMspropDbSchemaName		VARCHAR(400) = NULL,
													@NewMspropDbTableName		VARCHAR(400) = NULL,
													@Debug						CHAR(1) = 'N'
)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON

	DECLARE @Error				INT
	DECLARE @RowsAffected		INT
	DECLARE @StepTimeStart		DATETIME
	DECLARE @TimeStart			DATETIME


	-- Extract schema from cube create script and replace the source view to the view with new objects
	DECLARE @LineText							VARCHAR(3000)
	DECLARE @SchemaStartFound					BIT
	DECLARE @DbSchemaNamePos 					INT
	DECLARE @DbTableNamePos 					INT
	DECLARE @LineNumberObjectSchemaStart		INT
	DECLARE @ProjectDbSchemaNamePos				INT
	DECLARE @ProjectDbSchemaNameLength			INT
	DECLARE @ProjectDbTableNamePos				INT
	DECLARE @ProjectDbTableNameLength			INT

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point 1. Generate SSAS Object Schema For Cube Load XMLA Script   ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		SET @TimeStart = GETDATE()
		SET @StepTimeStart = GETDATE()
	END

	SELECT @LineNumberObjectSchemaStart = LineNumber
	FROM #ScriptCreateCube
	WHERE LineText LIKE '%msprop:DbTableName="' + @ProjectMspropDbTableName + '"%'

	IF @LineNumberObjectSchemaStart IS NULL
	BEGIN
		PRINT 'ERROR! Object with table name ' + @ProjectMspropDbTableName + ' is not found in SSAS DB definition.'
		RETURN 1 
	END
	
	DECLARE GetScriptTextCursor CURSOR FOR 
	SELECT LineText	FROM #ScriptCreateCube WHERE LineNumber >= @LineNumberObjectSchemaStart ORDER BY LineNumber

	OPEN GetScriptTextCursor

	SET @SchemaStartFound = 0
	WHILE (1=1)
	BEGIN
		FETCH FROM GetScriptTextCursor INTO @LineText	
		IF @@FETCH_STATUS != 0
			BREAK

		IF @SchemaStartFound = 0 AND @LineText NOT LIKE '%msprop:DbTableName="' + @ProjectMspropDbTableName + '"%'
			CONTINUE

		IF @SchemaStartFound = 0 
		BEGIN
			SET @SchemaStartFound = 1

			IF @NewMspropDbSchemaName IS NOT NULL AND @NewMspropDbTableName IS NOT NULL 
			BEGIN
				SELECT @DbSchemaNamePos = CHARINDEX ( 'msprop:DbSchemaName', @LineText ) 
				SELECT @ProjectDbSchemaNamePos = CHARINDEX ( '"', @LineText, @DbSchemaNamePos ) + 1
				SELECT @ProjectDbSchemaNameLength = CHARINDEX ( '"', @LineText, @ProjectDbSchemaNamePos + 1 ) - @ProjectDbSchemaNamePos

				SET @LineText = STUFF( @LineText, @ProjectDbSchemaNamePos, @ProjectDbSchemaNameLength, @NewMspropDbSchemaName )
				
				SELECT @DbTableNamePos = CHARINDEX ( 'msprop:DbTableName', @LineText ) 
				SELECT @ProjectDbTableNamePos = CHARINDEX ( '"', @LineText, @DbTableNamePos ) + 1
				SELECT @ProjectDbTableNameLength = CHARINDEX ( '"', @LineText, @ProjectDbTableNamePos + 1 ) - @ProjectDbTableNamePos
				
				SET @LineText = STUFF( @LineText, @ProjectDbTableNamePos, @ProjectDbTableNameLength, @NewMspropDbTableName )
			END
		END

		PRINT  @LineText

		IF @LineText LIKE '%</xs:complexType>%' -- need one more after that
		BEGIN
			PRINT  '                                        </xs:element>'
			BREAK
		END
	END

	CLOSE GetScriptTextCursor
	DEALLOCATE GetScriptTextCursor 

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point last. ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		PRINT '     Total time: ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
	END

	RETURN 0
END
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script]') AND type = 'P')
BEGIN
	PRINT 'CREATED PROCEDURE [ssasload].[Generate SSAS Object Schema For Cube Load XMLA Script]'
END
GO
