/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting Common SSAS load objects - [ssasload].[*] ' -- the objects include stored procedures, views etc. When tables are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('ssasload' ) AND obj.[type] IN ( 'U', 'P', 'SN', 'FN', 'IF', 'V' )
ORDER BY CASE obj.[type]  WHEN 'SN' THEN 0 WHEN 'V' THEN 1 WHEN 'U' THEN 2 ELSE 3 END  -- drop syns, then views as they can be defined WITH BINDDING
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	--PRINT @ObjectName + ' ' +  @ObjectType
	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'U' THEN 'TABLE'
												WHEN 'P' THEN 'PROCEDURE'
												WHEN 'SN' THEN 'SYNONYM'
												WHEN 'FN' THEN 'FUNCTION'
												WHEN 'IF' THEN 'FUNCTION'
												WHEN 'V' THEN 'VIEW'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	IF @ObjectSchema = 'gen' AND @ObjectType != 'SN' -- in gen schema only delete synonyms
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart'

DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ('ssasload')

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

PRINT 'Creating tables'

	-- Partitioned Measure Groups Attributes
CREATE TABLE [ssasload].[Measure Group]
(
	[Measure Group ID]														VARCHAR(200)	NOT NULL ,
	[Cube ID]																VARCHAR(200)	NOT NULL ,
	[Data Source ID]														VARCHAR(100)	NULL,		-- will be NULL for linked partitiones, where that require out-of-line bindings 
    [Is Single Partition]													BIT				NOT NULL,
	[Is Linked]																BIT				NOT NULL ,
	[Is Loaded In Full]														BIT				NOT NULL,	-- 1 means that the measure group is loaded in it's entirety as opposed to incremental load of new rows in separate partition
	[Number Of Effectived Days In Partition]								INT				NOT NULL DEFAULT 1,
	[Fact Table Schema]														VARCHAR(20)		NOT NULL  ,
	[Fact Table Name]														VARCHAR(200)	NOT NULL  ,
	[Fact Table Date Key Name]												VARCHAR(200)	NULL  ,
	[SSAS Source View Schema]												VARCHAR(20)		NOT NULL  ,
	[SSAS Source View Name]													VARCHAR(100)	NOT NULL  ,
	[SSAS Source View Date Key Name]										VARCHAR(200)	NULL  ,		-- can be NULL for non-date partitons, e.e, Role Mebership
	[SSAS Date View Schema]													VARCHAR(20)		NOT NULL  DEFAULT 'ssasdim',
	[SSAS Date View Name]													VARCHAR(100)	NOT NULL  DEFAULT 'Date',
	[SSAS Date View Date Column Name]										VARCHAR(100)	NOT NULL  DEFAULT 'Date',
	[SSAS Source View Where Clause To Filter On SSAS Server And Database]	VARCHAR(3000)	NULL , -- Special case for Measure group whose data is specific to the SSAS Senver and DB. Example of the dimension with this WHERE clause is security roles [ssasdim].[Role] where there are two columns with a SSAS server name and DB name. The view would include only rows for the SSAS server and database that match the SSAS DB that is being loaded. This flag will be used when constructing SSAS DB instance specific views	
)

ALTER TABLE [ssasload].[Measure Group]
ADD PRIMARY KEY CLUSTERED ( [Measure Group ID], [Cube ID] )

IF ( SELECT OBJECT_ID( '[ssasload].[Measure Group]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Measure Group]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Measure Group]'
END

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[ssasload].[Partition]'))
BEGIN
	CREATE TABLE [ssasload].[Partition]
	(
		SSASServerName						VARCHAR(100) NOT NULL 	,
		SSASDBName							VARCHAR(100) NOT NULL 	,

		-- Processing attributes
		PartitionNamePrefix					VARCHAR(200) NOT NULL,
		EffectiveDateStart					DATE NOT NULL,
		EffectiveDateEnd					DATE NOT NULL,
		PartitionNumber						INT NOT NULL,
		PartitionName						VARCHAR(400) NOT NULL,

		UpdatedOn							DATETIME   NOT NULL DEFAULT (GETDATE()),
	)
	
	ALTER TABLE [ssasload].[Partition]
	ADD PRIMARY KEY CLUSTERED ( SSASServerName, SSASDBName, PartitionNamePrefix, PartitionNumber )	
	
	IF ( SELECT OBJECT_ID( '[ssasload].[Partition]') ) IS NOT NULL
	BEGIN
		PRINT 'CREATED TABLE [ssasload].[Partition]'
	END
	ELSE
	BEGIN
		PRINT 'FAILED CREATING TABLE [ssasload].[Partition]'
	END
END
GO

-- objects for cube access
-- Configuration is a logical name that defines type of access that should be created.
-- Configunation is specified in ETL job. Because of that we need a master table with configuration names only
-- Example of configurations can be 'NFL Internet', etc"
-- Configuration is defined on a cube level. Multiple cubes with a database can have different configurations.
-- Each c6nfigurartion can have predefined rules associated with it' In that case the rules based access generated PnognammaticallY.
-- The configuration implies a set of roles that will be created for a given cube.
-- The [ssasload].[Access Configuration] table is populaied manuatly as part of Security Configuration

CREATE TABLE [ssasload].[Access Configuration]
(
	[Access Configuration Id]		INT IDENTITY NOT NULL PRIMARY KEY,
	[Access Configuration Name]	VARCHAR (100) NOT NULL UNIQUE	, -- Predefined name
	
	---- End of data
	[Updated On]			DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Access Configuration]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Access Configuration]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Access Configuration]'
END
GO

-- There wilt be multiple roles created for each Rule
-- Each Rule will have one or more Roles created for it. If more than one role is created for the rule all roles will have the same Prefix
-- This table is populated manually as part of the Security configuration
CREATE TABLE [ssasload].[Rule]
(
	[Rule Id]					INT IDENTITY NOT NULL PRIMARY KEY,
	[Rule Name]					VARCHAR (100) NOT NULL UNIQUE	,	-- Predefined rule name. Rule will trigger role generation
	[SSAS Role Name]			VARCHAR (100) NOT NULL UNIQUE	,	-- This name can be followed by user login .
	[Source Role Name]			VARCHAR (100) NULL,					-- Role name from system that is a source of roles 
	
	---- End of data
	[Updated On]			DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Rule]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Rule]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Rule]'
END
GO

-- This table is populated manually as part of the Security configuration
CREATE TABLE [ssasload].[Cube]
(
	[Cube Id]					INT IDENTITY NOT NULL PRIMARY KEY,
	[SSAS Cube Id]				VARCHAR (100) NOT NULL UNIQUE	,	-- Each SSAS Cube can be in different configuration, e.g., Prod and Dev
	[Access]					VARCHAR (100) NOT NULL DEFAULT 'Read'	,
	[Drillthrough]				VARCHAR (100) NOT NULL DEFAULT 'Drillthrough',
	[Process]					BIT NOT NULL DEFAULT 0,
	
	---- End of data
	[Updated On]			DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
GO

CREATE UNIQUE INDEX [ssasload Cube Indxl] ON [ssasload].[Cube] ( [SSAS Cube Id], [Access], [Drillthrough], [Process] )
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Cube]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Cube]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Cube]'
END
GO

-- This table includes dimension names and Ids of dimensions used in security roles.
-- It includes Ids of actual and role playing dimensions used to define security.
-- We need Cube Id here because the same name can be used in different cubes and it will correspond to different Id.
CREATE TABLE [ssasload].[Cube Dimension]
(
	[Cube Dimension Id]			INT IDENTITY NOT NULL PRIMARY KEY,
	[SSAS Cube Id]				VARCHAR (100) NOT NULL,
	[SSAS Dimension Id]			VARCHAR (100) NOT NULL,
	[SSAS Dimension Name]		VARCHAR (100) NOT NULL,

	---- End of data
	[Updated On]				DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]				VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
GO

CREATE UNIQUE INDEX [ssasload Cube Dimension Indxl] ON [ssasload].[Cube Dimension] ( [SSAS Cube Id], [SSAS Dimension Id] )
CREATE UNIQUE INDEX [ssasload Cube Dimension Indx2] ON [ssasload].[Cube Dimension] ( [SSAS Cube Id], [SSAS Dimension Name] )
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Cube Dimension]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Cube Dimension]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Cube Dimension]'
END
GO

-- This table is populated manually as part of the Security configuration
CREATE TABLE [ssasload].[Access Configuration Cube Rules And Roles]
(
	[Cube Configuration Rule Id]	INT IDENTITY NOT NULL PRIMARY KEY,
	[Access Configuration Id]				INT NOT NULL,
	[Cube Id]						INT NOT NULL,
	[Rule Id]						INT NULL,		-- Cubes within configuration can have rule based roles or roles created dynamically for a given configuration
	
	---- End of data
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Access Configuration Cube Rules And Roles]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Access Configuration Cube Rules And Roles]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Access Configuration Cube Rules And Roles]'
END
GO

-- This table is created dynamically for rule based roles or via UI.
-- When created via UI the UI will ask for cube, membership and dimension data related to this role
-- SELECT * FROM [ssasload].[Role]
CREATE TABLE [ssasload].[Role]
(
	[Role Id]						INT IDENTITY NOT NULL PRIMARY KEY,
	[Role Name]						VARCHAR (100) NOT NULL,
	[SSAS Server Name]				VARCHAR (100) NULL , -- all rule based roles will have [SSAS Server Name] and [SSAS DB Name] populated
	[SSAS DB Name]					VARCHAR(100) NULL , -- these two columns will be NULL for ad-hoc roles that should be processed each time a configunation is processed 

	---- End of data
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
CREATE UNIQUE INDEX [ssasload Role Indxl] ON [ssasload].[Role] ( [Role Name], [SSAS Server Name], [SSAS DB Name] )
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Role]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Role]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Role]'
END
GO

-- SELECT * FROM [ssasload].[Role Membership]
CREATE TABLE [ssasload].[Role Membership]
(
	[Role Membership Id]			INT IDENTITY NOT NULL PRIMARY KEY,
	[Role Id]						INT NULL,
	[Member]						VARCHAR (200) NULL,		-- The value for [Member] should be in domain\user_login format, e.g., dbtimes\victor

	---- End of data
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
CREATE UNIQUE INDEX [ssasload Role Membership Indxl] ON [ssasload].[Role Membership] ( [Role Id], [Member] )
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Role Membership]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Role Membership]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Role Membership]'
END
GO

-- SELECT * FROM [ssasload].[Role Cube]
CREATE TABLE [ssasload].[Role Cube]
(
	[Role Cube Id]			INT IDENTITY NOT NULL PRIMARY KEY,
	[Role Id]				INT NULL,
	[Cube Id]				INT NULL,

	---- End of data
	[Updated On]			DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
CREATE UNIQUE INDEX [ssasload Role Cube Indxl] ON [ssasload].[Role Cube] ( [Role Id], [Cube Id] )
GO

IF ( SELECT OBJECT_ID( '[ssasload].[Role Cube]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [ssasload].[Role Cube]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Role Cube]'
END
GO

-- SELECT * FROM [ssasload].[Role Dimension Data]
CREATE TABLE [ssasload].[Role Dimension Data]
(
	[Dimension Data Id]			INT IDENTITY NOT NULL PRIMARY KEY,
	[Role Id]					INT NOT NULL,
	[SSAS Dimension Name]		VARCHAR (100) NOT NULL,
	[SSAS Attribute Id]			VARCHAR (100) NOT NULL,
	[SSAS Attribute Name]		VARCHAR (100) NOT NULL,
	[Allowed Attribute Value]	VARCHAR (100) NULL, -- can be NULL if the Allowed set expression is supplied directly
	[Denied Attribute Value]	VARCHAR (100) NULL, -- can be NULL if the Denied set expnession is supplied dinectly
	[Allowed Set MDX]			VARCHAR (8000) NULL, -- can be null if the [A1lowed Attribute Value] is used
	[Denied Set MDX]			VARCHAR (8000) NULL, -- can be null if the [Denied Attnibute Value] is used
	[Default Member MDX]		VARCHAR (8000) NULL, 

	---- End of data
	[Updated On]			DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER,
)
CREATE UNIQUE INDEX [ssasload Dimension Data Indxl] ON [ssasload].[Role Dimension Data] ( [Role Id], [SSAS Dimension Name], [SSAS Attribute ID], [Allowed Attribute Value] ) WHERE [Allowed Attribute Value] IS NOT NULL
GO
CREATE UNIQUE INDEX [ssasload Dimension Data Indx2] ON [ssasload].[Role Dimension Data] ( [Role Id], [SSAS Dimension Name], [SSAS Attribute ID], [Denied Attribute Value] ) WHERE [Denied Attribute Value] IS NOT NULL
GO

-- This index enforces a single [Allowed Set MDX] and [Default Member MDX] expression per attribute
CREATE UNIQUE INDEX [ssasload Dimension Data Indx3] ON [ssasload].[Role Dimension Data] ( [Role Id], [SSAS Dimension Name], [SSAS Attribute ID] ) WHERE [Allowed Attribute Value] IS NULL AND [Denied Attribute Value] IS NULL
GO
IF ( SELECT OBJECT_ID( '[ssasload].[Role Dimension Data]') ) IS NOT NULL
BEGIN
	PRINT'CREATED TABLE [ssasload].[Role Dimension Data]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [ssasload].[Role Dimension Data]'
END

SET ANSI_PADDING OFF
PRINT 'Created common SSAS load tables in ' + DB_NAME() +' database'
