/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[gen].[Compare Tablular Results]') AND type IN (N'P', N'PC'))
BEGIN
	DROP PROCEDURE [gen].[Compare Tablular Results]
	PRINT 'DROPED PROCEDURE [gen].[Compare Tablular Results]'
END
GO

-- =============================================
-- Author:		Victor Shulman
-- =============================================
CREATE PROCEDURE [gen].[Compare Tablular Results] (	@SSASConnectionStringLeft		VARCHAR(300),
													@SSASConnectionStringRight		VARCHAR(300),
													@AliasLeft						VARCHAR(300),
													@AliasRight						VARCHAR(300),
													@QueryLeft						VARCHAR(MAX),
													@QueryRight						VARCHAR(MAX),
													@TablularResultSetColumnsTypes	VARCHAR(MAX),	-- It is the same for both left and right queries
													@UniqueKeyColumns				VARCHAR(MAX),	-- We must have unique key - it is required to find out which data is missing
													@ColumnsToExcludeFromCompare	VARCHAR(MAX),	-- Only key columns are used in compare. Non key columns are excluded 
													@FloatCompareThreshold			FLOAT = 0,
													@Debug							CHAR(1) = 'N' 
)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON

	DECLARE @RowsAffected					INT
	DECLARE @StepTimeStart					DATETIME
	DECLARE @TimeStart						DATETIME
	DECLARE @ProcessStarted					DATETIME = GETDATE()

	DECLARE @Statement						NVARCHAR(MAX);


    DECLARE @Message						NVARCHAR(4000);


	-- Step 1. Create tables and collections of columns for compare
	
	-- 1.1 Tables
	CREATE TABLE #NotKeysColumnsToExcludeFromCompare
	(
		[Column Name]				VARCHAR(200) NOT NULL,
	)
	CREATE TABLE #UniqueKeyColumns
	(
		[Column Name]				VARCHAR(200) NOT NULL,
	)
	CREATE TABLE #NotKeysColumns
	(
		[Column Name]				VARCHAR(200) NOT NULL,
	)
	CREATE TABLE #TabularResultSetColumns
	(
		[Column Name]				VARCHAR(200) NOT NULL,
		[Is Float Type]				BIT NOT NULL
	)
	
	DECLARE @NonKeyColumns VARCHAR(MAX)
	
	SET @SSASConnectionStringLeft	= ISNULL( NULLIF( @SSASConnectionStringLeft	, '' ), '' )
	SET @SSASConnectionStringRight	= ISNULL( NULLIF( @SSASConnectionStringRight, '' ), '' )

	SET @AliasLeft	= CASE WHEN NULLIF( @AliasLeft, '' ) IS NULL THEN 'Left' ELSE @AliasLeft END
	SET @AliasRight = CASE WHEN NULLIF( @AliasRight, '' ) IS NULL THEN 'Right' ELSE @AliasRight END

	DECLARE @IsMdxQueryLeft		CHAR(1) = 'Y'
	DECLARE @IsMdxQueryRight	CHAR(1) = 'Y'
	IF @SSASConnectionStringLeft = ''
		SET @IsMdxQueryLeft = 'N'
	IF @SSASConnectionStringRight = ''
		SET @IsMdxQueryRight = 'N'

	SELECT @Statement = N'
		DECLARE @NEW_LINE	CHAR(2) = CHAR(13) + CHAR(10)

		-- Extract all column names from the list of columns
		INSERT #TabularResultSetColumns
		SELECT	''['' + colNamesSplit.Item + '']''		AS [Column Name], 
				CASE WHEN REPLACE( colNamesTypes.Item, ''['' + colNamesSplit.Item + '']'', '''') LIKE ''%FLOAT%''
						THEN 1 
						ELSE 0
						END								AS [Is Float Type]
		FROM [gen].[SPLIT]( ''<@TablularResultSetColumnsTypes>'', '','' )	AS colNamesTypes
			CROSS APPLY  [gen].SPLIT( colNamesTypes.Item, '']'' ) AS colNamesTypesSplit
			CROSS APPLY  [gen].SPLIT( colNamesTypesSplit.Item, ''['' ) AS colNamesSplit
		WHERE		colNamesTypes.Item != @NEW_LINE		-- Remove last new line that resulted from last comma
				AND colNamesSplit.ItemNumber % 2 = 0

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 1. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''table #TabularResultSetColumns'' AS [table #TabularResultSetColumns], 
						* 
				FROM #TabularResultSetColumns
		END

		-- Extract Unique column names from the list of uniue key columns
		-- Thi statement is exatly as above only has @UniqueKeyColumns instead of @TablularResultSetColumnsTypes,  
		-- it is seleted into another temp table name and does not have [Is Float Type] column
		INSERT #UniqueKeyColumns
		SELECT ''['' + colNamesSplit.Item + '']'' AS [Column Name]
		FROM [gen].[SPLIT]( ''<@UniqueKeyColumns>'', '','' ) AS colNamesTypes
			CROSS APPLY  [gen].SPLIT( colNamesTypes.Item, '']'' ) AS colNamesTypesSplit
			CROSS APPLY  [gen].SPLIT( colNamesTypesSplit.Item, ''['' ) AS colNamesSplit
		WHERE		colNamesTypes.Item != @NEW_LINE	-- Remove line caused by last comma
				AND colNamesSplit.ItemNumber % 2 = 0 -- exclude lines that have other parts of the column list

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 2. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''#UniqueKeyColumns'' AS [#UniqueKeyColumns], * FROM #UniqueKeyColumns
		END

		-- Extract column names from the list of columns to exclude from compare
		-- Thi statement is exatly as above only has @ColumnsToExcludeFromCompare instead of @TablularResultSetColumnsTypes and diffferent temp table name and no [Is Float Type] column
		IF ''<@ColumnsToExcludeFromCompare>'' != ''''
			INSERT #NotKeysColumnsToExcludeFromCompare
			SELECT -- colNamesTypes.ItemNumber, colNamesTypes.Item, colNamesTypesSplit.Item, colNamesTypesSplit.ItemNumber, colNamesSplit.ItemNumber, 
					''['' + colNamesSplit.Item + '']'' AS [Column Name]
			FROM [gen].[SPLIT]( ''<@ColumnsToExcludeFromCompare>'', '','' ) AS colNamesTypes
				CROSS APPLY  [gen].SPLIT( colNamesTypes.Item, '']'' ) AS colNamesTypesSplit
				CROSS APPLY  [gen].SPLIT( colNamesTypesSplit.Item, ''['' ) AS colNamesSplit
			WHERE		colNamesTypes.Item != @NEW_LINE	-- Remove line caused by last comma
					AND colNamesSplit.ItemNumber % 2 = 0 -- exclude lines that have other parts of the column list

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 3. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''#NotKeysColumnsToExcludeFromCompare'' AS [#NotKeysColumnsToExcludeFromCompare], * FROM #NotKeysColumnsToExcludeFromCompare
		END

		-- Create a table with all colums that are not keys
		INSERT #NotKeysColumns
		SELECT [Column Name]
		FROM #TabularResultSetColumns
		EXCEPT
		(
			SELECT [Column Name]
			FROM #UniqueKeyColumns
			UNION
			SELECT [Column Name]
			FROM #NotKeysColumnsToExcludeFromCompare
		)

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 4. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''table #NotKeysColumns'' AS [table #NotKeysColumns], * FROM #NotKeysColumns
		END
	'

	SELECT @Statement = REPLACE(@Statement, '<@TablularResultSetColumnsTypes>',				@TablularResultSetColumnsTypes					);
	SELECT @Statement = REPLACE(@Statement, '<@UniqueKeyColumns>',			@UniqueKeyColumns			);
	SELECT @Statement = REPLACE(@Statement, '<@ColumnsToExcludeFromCompare>',	ISNULL( @ColumnsToExcludeFromCompare, '' ) );
	SELECT @Statement = REPLACE(@Statement, '<@Debug>', ISNULL( @Debug, 'N' ) );

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point 5. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		SET @StepTimeStart = GETDATE();

		PRINT CAST( SUBSTRING( @Statement, 1,	  8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 8001,  8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 16001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 24001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 32001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 40001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 48001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 56001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 64001, 8000 ) AS VARCHAR(8000));

		RAISERROR ( 'Point 6', 0, 1) WITH NOWAIT
	END

	EXECUTE sp_executesql @Statement, N' @StepTimeStart DATETIME, @NonKeyColumns VARCHAR(MAX) OUT', @StepTimeStart = @StepTimeStart, @NonKeyColumns = @NonKeyColumns OUT


		-- Create a comma delimited list on non-key columns
	SELECT @NonKeyColumns = COALESCE(@NonKeyColumns, '') + [Column Name] + ','
	FROM #NotKeysColumns
	SELECT @NonKeyColumns = REVERSE( SUBSTRING( REVERSE( @NonKeyColumns ), 2, 1000000000000000000 ) ) 

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point 7. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		SET @StepTimeStart = GETDATE();

		IF ( @Debug != 'P' )
			SELECT @NonKeyColumns AS [@NonKeyColumns]
	END

	-- Step 2. Run queries and compare results
	SELECT @Statement = N'
		DECLARE @Statement					NVARCHAR(MAX);

		CREATE TABLE #Left
		(
			<@TablularResultSetColumnsTypes>
		)
		CREATE TABLE #Right
		(
			<@TablularResultSetColumnsTypes>
		)

		IF ''<@IsMdxQueryLeft>'' = ''Y''
			INSERT INTO #Left
			EXEC [api].[SelectViaMdx] ''<@SSASConnectionStringLeft>'', ''<@QueryLeft>'', ''<@TablularResultSetColumnsTypes>'' 
		ELSE
		BEGIN
			SET @Statement = N''<@QueryLeft>''
			INSERT INTO #Left
			EXEC sp_executesql @Statement
		END

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 8. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''table #Left'' AS [table #Left], * 
				FROM #Left
		END

		IF ''<@IsMdxQueryRight>'' = ''Y''
			INSERT INTO #Right
			EXEC [api].[SelectViaMdx] ''<@SSASConnectionStringRight>'', ''<@QueryRight>'', ''<@TablularResultSetColumnsTypes>'' 
		ELSE
		BEGIN
			SET @Statement = N''<@QueryRight>''
			INSERT INTO #Right
			EXEC sp_executesql @Statement
		END

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 9. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''table #Right'' AS [Table #Right], * 
				FROM #Right
		END

		-- Create Left keys
		SELECT REPLACE( REPLACE( 
		( SELECT	<@UniqueKeyColumns>	FOR XML RAW	), ''<row '', '''' ), ''/>'', '''' ) AS [Unique Key Concat],
		*
		INTO #LeftWithConcatenatedKey
		FROM #Left

		-- Create Right keys
		SELECT REPLACE( REPLACE( 
		( SELECT	<@UniqueKeyColumns>	FOR XML RAW	), ''<row '', '''' ), ''/>'', '''' ) AS [Unique Key Concat],
		*
		INTO #RightWithConcatKey
		FROM #Right

		-- Create the result of compare
		SELECT	CAST( [Unique Key Concat] AS VARCHAR(MAX))	AS [Unique Key Columns]	,
				<@UniqueKeyColumns>		,
				CAST( NULL				AS VARCHAR(200) )	AS [Column Name]		,
				CAST( NULL				AS VARCHAR(200) )	AS [<@AliasLeft> Value]			,
				CAST( NULL				AS VARCHAR(200) )	AS [<@AliasRight> Value]		,
				CAST( ''In <@AliasLeft> Only''	AS VARCHAR(200) )	AS [Message]
		INTO #CompareResult
		FROM #LeftWithConcatenatedKey AS leftWithKey
		WHERE NOT EXISTS (	SELECT 1 
							FROM #RightWithConcatKey 
							WHERE [Unique Key Concat] = leftWithKey.[Unique Key Concat] )

		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 10. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''#CompareResult after In <@AliasLeft> Only'' AS [CompareResult after In <@AliasLeft> Only], * FROM #CompareResult
		END

		INSERT #CompareResult
		SELECT	[Unique Key Concat]		AS [Unique Key Columns]	,
				<@UniqueKeyColumns>		,
				NULL					AS [Column Name]		,
				NULL					AS [<@AliasLeft> Value]			,
				NULL					AS [<@AliasRight> Value]		,
				''In <@AliasRight> Only''		AS [Message]
		FROM #RightWithConcatKey AS rightWithKey
		WHERE NOT EXISTS ( SELECT 1 FROM #LeftWithConcatenatedKey WHERE [Unique Key Concat] = rightWithKey.[Unique Key Concat] )


		IF ( ''<@Debug>'' != ''N'' )
		BEGIN 
			PRINT ''Point 11. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			IF ( ''<@Debug>'' != ''P'' )
				SELECT ''#CompareResult after In <@AliasRight> Only'' AS [CompareResult after In <@AliasRight> Only], * FROM #CompareResult
		END


		-- Go through all Not Key columns and compare
		-- TODO. For floating numbers threshold the compare within need to be done using dynamic SQL 
		-- because the column name will change each time 

		DECLARE @CompareStatement					NVARCHAR(MAX);
		DECLARE @UniqueKeyColumnsWithTableAlias		NVARCHAR(MAX);

		SELECT @UniqueKeyColumnsWithTableAlias = REPLACE( ''<@UniqueKeyColumns>'', ''['', ''leftWithKey.['' ) -- leftWithKey or rightWithKey can both be used because it is inner join
		DECLARE @ColumnName				VARCHAR(200)
		DECLARE @IsFloatColumnType		BIT

		DECLARE NonKeyColumnsCursor CURSOR LOCAL FOR 
		SELECT cols.[Column Name]	,
			   cols.[Is Float Type]	
		FROM #NotKeysColumns AS notKeyCols
			INNER JOIN #TabularResultSetColumns AS cols ON notKeyCols.[Column Name] = cols.[Column Name]
		OPEN NonKeyColumnsCursor

		WHILE (1=1)
		BEGIN
			FETCH FROM NonKeyColumnsCursor INTO @ColumnName, @IsFloatColumnType
			IF @@FETCH_STATUS != 0
				BREAK

			PRINT ''Comparing values of '' + @ColumnName + '' column''

			SELECT @ColumnName = REPLACE( @ColumnName, ''['', '''' )
			SELECT @ColumnName = REPLACE( @ColumnName, '']'', '''' )

			IF @IsFloatColumnType = 1
				SELECT @CompareStatement = N''
							INSERT #CompareResult
							SELECT	leftWithKey.[Unique Key Concat]											AS [Unique Key Columns]	,
									<@UniqueKeyColumnsWithTableAlias> ,
									''''<@ColumnName>''''													AS [Column Name]		,
									CAST( STR( leftWithKey.[<@ColumnName>]	, 30, 10 ) AS VARCHAR(200))		AS [<@AliasLeft> Value]	,
									CAST( STR( rightWithKey.[<@ColumnName>] , 30, 10 ) AS VARCHAR(200))		AS [<@AliasRight> Value],
									''''Not Equal''''														AS [Message]
							FROM #RightWithConcatKey AS rightWithKey
								INNER JOIN #LeftWithConcatenatedKey AS leftWithKey ON leftWithKey.[Unique Key Concat] = rightWithKey.[Unique Key Concat]
							WHERE ABS( ISNULL( leftWithKey.[<@ColumnName>], 0 ) - ISNULL( rightWithKey.[<@ColumnName>], 0 ) ) > <@FloatCompareThresholdColumn>
						''
			ELSE
				SELECT @CompareStatement = N''
							INSERT #CompareResult
							SELECT	leftWithKey.[Unique Key Concat]								AS [Unique Key Columns]	,
									<@UniqueKeyColumnsWithTableAlias> ,
									''''<@ColumnName>''''										AS [Column Name]		,
									ISNULL( CAST( leftWithKey.[<@ColumnName>] AS  NVARCHAR(200)), ''''NULL'''' )			AS [<@AliasLeft> Value]			,
									ISNULL( CAST( rightWithKey.[<@ColumnName>] AS NVARCHAR(200)), ''''NULL'''' )			AS [<@AliasRight> Value]		,
									''''Not Equal''''											AS [Message]
							FROM #RightWithConcatKey AS rightWithKey
								INNER JOIN #LeftWithConcatenatedKey AS leftWithKey ON leftWithKey.[Unique Key Concat] = rightWithKey.[Unique Key Concat]
							WHERE		( leftWithKey.[<@ColumnName>] IS NULL AND rightWithKey.[<@ColumnName>] IS NOT NULL )
									OR	( leftWithKey.[<@ColumnName>] IS NOT NULL AND rightWithKey.[<@ColumnName>] IS NULL )
									OR	( leftWithKey.[<@ColumnName>] IS NOT NULL AND rightWithKey.[<@ColumnName>] IS NOT NULL 
											AND leftWithKey.[<@ColumnName>] != rightWithKey.[<@ColumnName>] )
						''

			SELECT @CompareStatement = REPLACE(@CompareStatement, ''<@ColumnName>'',	@ColumnName	);
			SELECT @CompareStatement = REPLACE(@CompareStatement, ''<@UniqueKeyColumnsWithTableAlias>'', @UniqueKeyColumnsWithTableAlias );
			SELECT @CompareStatement = REPLACE(@CompareStatement, ''<@FloatCompareThresholdColumn>'', ''<@FloatCompareThreshold>'' );

			IF ( ''<@Debug>'' != ''N'' )
			BEGIN 
				PRINT ''Point 12. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
				SET @StepTimeStart = GETDATE();

				PRINT CAST( SUBSTRING( @CompareStatement, 1,	  8000 ) AS VARCHAR(8000));
			END

		EXECUTE sp_executesql @CompareStatement
		END

		CLOSE NonKeyColumnsCursor
		DEALLOCATE NonKeyColumnsCursor 		


		-- Select result
		ALTER TABLE #CompareResult
		DROP COLUMN [Unique Key Columns]

		SELECT	* 
		FROM #CompareResult

		SELECT	@RowsAffected = @@ROWCOUNT
	'

	SELECT @Statement = REPLACE(@Statement, '<@SSASConnectionStringLeft>'		, @SSASConnectionStringLeft	);
	SELECT @Statement = REPLACE(@Statement, '<@SSASConnectionStringRight>'		, @SSASConnectionStringRight);
	SELECT @Statement = REPLACE(@Statement, '<@AliasLeft>'						, @AliasLeft 				);
	SELECT @Statement = REPLACE(@Statement, '<@AliasRight>'						, @AliasRight				);
	SELECT @Statement = REPLACE(@Statement, '<@QueryLeft>'						, REPLACE( @QueryLeft, '''', '''''' ) );
	SELECT @Statement = REPLACE(@Statement, '<@QueryRight>'						, REPLACE( @QueryRight, '''', '''''' ) );
	SELECT @Statement = REPLACE(@Statement, '<@TablularResultSetColumnsTypes>'	, @TablularResultSetColumnsTypes	);
	SELECT @Statement = REPLACE(@Statement, '<@UniqueKeyColumns>'				, @UniqueKeyColumns			);
	SELECT @Statement = REPLACE(@Statement, '<@NonKeyColumns>'					, @NonKeyColumns			);
	SELECT @Statement = REPLACE(@Statement, '<@IsMdxQueryLeft>'					, @IsMdxQueryLeft			);
	SELECT @Statement = REPLACE(@Statement, '<@IsMdxQueryRight>'				, @IsMdxQueryRight			);
	SELECT @Statement = REPLACE(@Statement, '<@FloatCompareThreshold>'			, CAST( ABS( @FloatCompareThreshold ) AS VARCHAR( 50 ) ) );
	SELECT @Statement = REPLACE(@Statement, '<@Debug>'							, ISNULL( @Debug, 'N' )		);

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point 13. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
		SET @StepTimeStart = GETDATE();

		PRINT CAST( SUBSTRING( @Statement, 1,	  8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 8001,  8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 16001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 24001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 32001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 40001, 8000 ) AS VARCHAR(8000));
		PRINT CAST( SUBSTRING( @Statement, 48001, 8000 ) AS VARCHAR(8000));

		RAISERROR ( 'Point 14', 0, 1) WITH NOWAIT
	END

	EXECUTE sp_executesql @Statement, N' @StepTimeStart DATETIME, @RowsAffected INT OUT', @StepTimeStart = @StepTimeStart, @RowsAffected = @RowsAffected OUT

	IF ( @Debug != 'N' )
	BEGIN 
		PRINT 'Point last. End Compare Tablular Results  ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		PRINT '     Total time: ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
	END

	RETURN CASE WHEN @RowsAffected > 0 THEN 1 ELSE 0 END
END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[gen].[Compare Tablular Results]') AND type IN (N'P', N'PC'))
BEGIN
	PRINT 'CREATED PROCEDURE [gen].[Compare Tablular Results]'
END
GO
