/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[gen].[Get Changed Date And Last Version]') AND type in (N'P', N'PC'))
BEGIN
	DROP PROCEDURE [gen].[Get Changed Date And Last Version]
	PRINT 'DROPED PROCEDURE [gen].[Get Changed Date And Last Version]'
END
GO

CREATE PROCEDURE [gen].[Get Changed Date And Last Version] (	@SourceServerName			VARCHAR(100), -- name of a linked server with Source DB, when NULL or empty string the local server will be used
																@SourceDBName				VARCHAR(100), -- name of Source DB
																@TableSchema				VARCHAR(20),
																@TableName					VARCHAR(100),
																@EffectiveDateColumnName    VARCHAR(100),
																@LastProcessedVersion		BIGINT,     
																@MaxVersion					BIGINT OUTPUT,
																@EarliestChangedDate		DATE OUTPUT,
																@Debug						CHAR(1) = 'N' )

AS
BEGIN
---------------------------------------------------------------------------------------------------------------------
-- Returns max SYS_CHANGE_VERSION for a given table with Minimun Effective date (This depends on primary col name for each table)
-- Returns NULL If the table is not monitored or column name is not valid.
-- Tables monitored:
--                          	[core].[CORE_AGENCY_PASSTHRU_HISTORY]
--		[core].[CORE_SAA_POSITION_HISTORY]
--		[excel].[WHOLE_LOAN_HISTORY]
--		[finance].[NA_TRR_ADJUSTMENT]
--		[opics].[OPICS_AFS_HISTORY]
--		[opics].[OPICS_MTM_HISTORY]
--		[pldb].[PLDB_DERIVATIVE_RISK_HISTORY]
--		[pldb].[PLDB_SECURITY_RISK_HISTORY]
--		[pldb].[RVGUSD_DEBT_HISTORY]
--		[poplar].[poplar_feed]
--		[tfa].[TFA_MASTER]
--		pldb.SECURITY_HISTORY
---------------------------------------------------------------------------------------------------------------------
	SET NOCOUNT ON

	DECLARE @Error				INT
	DECLARE @RowsAffected		INT
	DECLARE @StepTimeStart		DATETIME
	DECLARE @TimeStart			DATETIME

	DECLARE	@TableAndColumnExists	INT
   
	DECLARE @Statement			NVARCHAR(MAX);
	DECLARE @RemoteStatement	NVARCHAR(MAX);
	
	DECLARE @ThreePartTableName	VARCHAR(120)

	DECLARE @Message			NVARCHAR(4000);
	
	DECLARE @ProcessStarted		DATETIME = GETDATE()

	BEGIN TRY

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 1. Start determining Changed Date And Last Version      ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
			SET @TimeStart = GETDATE()
			SET @StepTimeStart = GETDATE()
		END

		SET @MaxVersion	= NULL
		SET @EarliestChangedDate = NULL	

		IF NULLIF( @SourceDBName, '' ) IS NULL
			SET @SourceDBName = DB_NAME()

		SET @ThreePartTableName = '[' + @SourceDBName + '].[' + @TableSchema + '].[' + @TableName + ']'

		SET @RemoteStatement = 'SELECT * FROM OPENQUERY( [' + @SourceServerName + '], ''EXEC ( ''''<@LocalServerStatement>'''' )''' + ')'

		-- Check if the table and col exists
		SET @Statement =   'SELECT 1 FROM [' + @SourceDBName + '].sys.columns WHERE name = '''  + @EffectiveDateColumnName  + ''''  + ' AND object_id = OBJECT_ID( ''' + @ThreePartTableName + ''' )'

		IF @SourceServerName != '' -- remote server
			SELECT @Statement = REPLACE( @RemoteStatement, '<@LocalServerStatement>', REPLACE( @Statement, '''', '''''''''' ) )
		-- Place the result into variable
		SELECT @Statement = 'SET @TableAndColumnExists = (' + @Statement + ')'

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 2. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			PRINT @Statement;
		END

		EXECUTE  sp_executesql @Statement,N'@TableAndColumnExists INT OUTPUT', @TableAndColumnExists  OUTPUT

		IF @TableAndColumnExists IS NULL
			RETURN 1

	-- Check if the table is monitored
		SET @Statement =   'SELECT 1 FROM [' + @SourceDBName + '].sys.change_tracking_tables WHERE object_id= OBJECT_ID( ''' + @ThreePartTableName + ''' )'
		IF @SourceServerName != '' -- remote server
			SELECT @Statement = REPLACE( @RemoteStatement, '<@LocalServerStatement>', REPLACE( @Statement, '''', '''''''''' ) )
		-- Place the result into variable
		SELECT @Statement = 'SET @TableAndColumnExists = (' + @Statement + ')'

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 3. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			PRINT @Statement;
		END

		EXECUTE  sp_executesql @Statement, N'@TableAndColumnExists INT OUTPUT', @TableAndColumnExists  OUTPUT

		IF @TableAndColumnExists IS NULL
			RETURN 1

		SET @Statement =   'SELECT MAX(SYS_CHANGE_VERSION) FROM CHANGETABLE(CHANGES ' + @ThreePartTableName +', ' + CONVERT(VARCHAR(10), @LastProcessedVersion) + ' ) AS ct '
		IF @SourceServerName != '' -- remote server
			SELECT @Statement = REPLACE( @RemoteStatement, '<@LocalServerStatement>', REPLACE( @Statement, '''', '''''''''' ) )
		-- Place the result into variable
		SELECT @Statement = 'SET @MaxVersion = (' + @Statement + ')'
	
		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 5. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			PRINT @Statement;
		END

		EXEC  sp_executesql @Statement, N'@MaxVersion BIGINT OUTPUT', @MaxVersion  OUTPUT
	
--		SET @Statement =   'SELECT * FROM OPENQUERY( [' + @SourceServerName + '],''EXEC '+ '(' + '''''' +' USE [' + @SourceDBName + '] ;' + 'SELECT   MIN(' + @EffectiveDateColumnName + ')  FROM CHANGETABLE(CHANGES ['+@SourceDBName + '].' + @ThreePartTableName+', ' + convert(varchar(10), @LastProcessedVersion) + ' ) CT ' + '''''' + ')'')'
		SET @Statement =   'SELECT MIN([' + @EffectiveDateColumnName + '])  FROM CHANGETABLE(CHANGES ' + @ThreePartTableName +', ' + CONVERT(VARCHAR(10), @LastProcessedVersion) + ' ) AS ct'
		IF @SourceServerName != '' -- remote server
			SELECT @Statement = REPLACE( @RemoteStatement, '<@LocalServerStatement>', REPLACE( @Statement, '''', '''''''''' ) )
			-- Place the result into variable
		SELECT @Statement = 'SET @EarliestChangedDate = (' + @Statement + ')'

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 6. ' + CONVERT( VARCHAR(6), DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			PRINT @Statement;
		END

		EXEC sp_executesql @Statement, N'@EarliestChangedDate DATE OUTPUT', @EarliestChangedDate OUTPUT
		
		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point last. End determining Changed Date And Last Version  ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
			PRINT '     Total time: ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		END


		RETURN 0
	END TRY
	BEGIN CATCH
	    DECLARE @ErrorMessage NVARCHAR(4000);
		DECLARE @ErrorSeverity INT;
		DECLARE @ErrorState INT;

		SELECT	@ErrorMessage  =
					  '   ERROR_NUMBER: ' + ISNULL( CONVERT(VARCHAR(6),ERROR_NUMBER()),'')  + CHAR(13)
					+ '   ERROR_SEVERITY: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_SEVERITY()),'')  + CHAR(13)
					+ '   ERROR_STATE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_STATE()),'')  + CHAR(13)
					+ '   ERROR_PROCEDURE: '  + ISNULL( ERROR_PROCEDURE(),'')  + CHAR(13)
					+ '   ERROR_LINE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_LINE()),'') + CHAR(13)
					+ '   ERROR_MESSAGE: '  + ISNULL( ERROR_MESSAGE(),'')  + CHAR(13),
				@ErrorSeverity = ERROR_SEVERITY(),
				@ErrorState = ERROR_STATE();

		PRINT 'Last statement: ' + CHAR(13)
				+ @Statement 
		PRINT @ErrorMessage
	
		SET @MaxVersion	= NULL
		SET @EarliestChangedDate = NULL	

		-- Log and handle ERROR, because the table may not be monitored

		RETURN 1
	END CATCH
          
END
go
SET ANSI_NULLS OFF
go
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[gen].[Get Changed Date And Last Version]') AND type IN (N'P', N'PC'))
BEGIN
	PRINT 'CREATED PROCEDURE [gen].[Get Changed Date And Last Version]'
END
GO

