/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sample].[Pre-Process Source]') AND type IN (N'P', N'PC'))
BEGIN
	DROP PROCEDURE [sample].[Pre-Process Source]
	PRINT 'DROPED PROCEDURE [sample].[Pre-Process Source]'
END
GO

-- =============================================
-- Author:		Victor Shulman
--	The assumptions:
--		* all tables will have [Id] as unique key.
--			If that is not the case either pass the column name as parameter or 
--			do other modifications to accomodate wider differences between source tables
--		* the table with data has the same schema and name as the actual table with ' Temp' suffix
--	[sample].[Pre-Process Source] 'sample','Some Source Data', 'Y'
-- =============================================
CREATE PROCEDURE [sample].[Pre-Process Source] (	@DestTableSchema	VARCHAR (255) ,
													@DestTableName		VARCHAR (255) ,
													@Debug				CHAR (1) = 'N'
												)
AS
BEGIN
-- SET NOCOUNT ON added to prevent extra result sets from interfering with SELECT statements.
	SET NOCOUNT ON

	BEGIN TRY;
		DECLARE @Error				INT
		DECLARE @StepTimestart		DATETIME
		DECLARE @Timestart			DATETIME = GETDATE( )
		DECLARE @ProcessStarted		DATETIME = GETDATE( )
		DECLARE @Message			NVARCHAR(4000);
		DECLARE @Result				INT
		DECLARE @Statement			NVARCHAR(MAX);
		DECLARE @RowsAffected		INT
		DECLARE @NEW_LINE_CHAR		NCHAR(2) = NCHAR(13) + NCHAR(10)

		DECLARE @PrevEffectiveDate			DATE = NULL
		DECLARE @DestTable					VARCHAR ( 500 )
		DECLARE @DestTableTemp				VARCHAR ( 500 )
		DECLARE @TableColumns				VARCHAR(MAX) = ''
		DECLARE @TablecolumnsForRowHash		VARCHAR(MAX) = ''
		DECLARE @MAX_DATA_LENGTH_FOR_HASH	INT = 510 -- This is the max length of the column before using HASHBYTES to compress the column into a hash in order to satisfy max input size of the HASHBYTES for the row

		PRINT '*** STARTED Pre-Process Source at ' + RTRIM( CONVERT(CHAR(40), GETDATE(), 8) ) + @NEW_LINE_CHAR

		SET @Message = '[sample].[Pre-Process Source] parameters: ' + @NEW_LINE_CHAR
						+ '@DestTableSchema - ' + ISNULL(@DestTableSchema, 'NULL' ) + @NEW_LINE_CHAR
						+ '@DestTableName   - ' + ISNULL(@DestTableName , 'NULL' ) + @NEW_LINE_CHAR
						+ '@Debug           - ' + ISNULL(@Debug , 'NULL' ) + @NEW_LINE_CHAR + @NEW_LINE_CHAR
		PRINT @Message

		SET @DestTable = QUOTENAME( @DestTableSchema ) + '.' + QUOTENAME( @DestTableName )
		SET @DestTableTemp = QUOTENAME( @DestTableSchema ) + '.' + QUOTENAME( @DestTableName + ' Temp' )


		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 1. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
		END

		-- Create temp table without the non-data columns to load the the source data
		SELECT @Statement =
				N'
					IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N''<@DestTableTemp>''))
					BEGIN
						DROP TABLE <@DestTableTemp>
						PRINT ''DROPPED TABLE <@DestTableTemp>''
					END

					SELECT *
					INTO <@DestTableTemp>
					FROM <@DestTable>
					WHERE 1 = 0 

					ALTER TABLE <@DestTableTemp>	DROP COLUMN [RowId]					
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Created On]			
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Created By]			
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Version]				
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Effective Date Start]	
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Effective Date End]	
					ALTER TABLE <@DestTableTemp>	DROP COLUMN	[Row Hash]				

					IF ( SELECT OBJECT_ID( ''<@DestTableTemp>'') ) IS NOT NULL
					BEGIN
						PRINT ''CREATED TABLE <@DestTableTemp>''
					END
					ELSE
					BEGIN
						PRINT ''FAILED CREATING TABLE <@DestTableTemp>''
					END

				'
		SELECT @Statement = REPLACE(@Statement, '<@DestTableTemp>', @DestTableTemp );
		SELECT @Statement = REPLACE(@Statement, '<@DestTable>', @DestTable );

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 2. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			PRINT 'Statement to create temp table'
			PRINT CAST ( @Statement AS TEXT )
		END
		EXECUTE sp_executesql @Statement

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point last. ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		END	

		PRINT '*** COMPLETED Pre-Process Source in ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + ' sec at ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8)) + @NEW_LINE_CHAR
		
		RETURN 0

	END TRY
	BEGIN CATCH
		DECLARE @ErrorMessage		NVARCHAR(4000)
		DECLARE @ErrorSeverity		INT
		DECLARE @ErrorState			INT 
		
		SELECT	@ErrorMessage  =
					  '   ERROR_NUMBER: ' + ISNULL( CONVERT(VARCHAR(6),ERROR_NUMBER()),'')  + CHAR(13)
					+ '   ERROR_SEVERITY: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_SEVERITY()),'')  + CHAR(13)
					+ '   ERROR_STATE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_STATE()),'')  + CHAR(13)
					+ '   ERROR_PROCEDURE: '  + ISNULL( ERROR_PROCEDURE(),'')  + CHAR(13)
					+ '   ERROR_LINE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_LINE()),'') + CHAR(13)
					+ '   ERROR_MESSAGE: '  + ISNULL( ERROR_MESSAGE(),'')  + CHAR(13),
				@ErrorSeverity = ERROR_SEVERITY(),
				@ErrorState = ERROR_STATE();
	
		IF @@TRANCOUNT > 0
			ROLLBACK TRANSACTION

	    RAISERROR ( @ErrorMessage, @ErrorSeverity, @ErrorState );

		RETURN 1
	END CATCH

END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sample].[Pre-Process Source]') AND type IN (N'P', N'PC'))
BEGIN
	PRINT 'CREATED PROCEDURE [sample].[Pre-Process Source]'
END
GO
