/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stg].[Post Process Source]') AND type IN (N'P', N'PC'))
BEGIN
	DROP PROCEDURE [stg].[Post Process Source]
	PRINT 'DROPED PROCEDURE [stg].[Post Process Source]'
END
GO

-- =============================================
-- Author:		Victor Shulman
--	The assumptions:
--		* all tables will have [<@DestTableUniqueColumnName>] as unique key.
--			If that is not the case either pass the column name as parameter or 
--			do other modifications to accomodate wider differences between source tables
--		* the table with data has the same schema and name as the actual table with ' Temp' suffix
--	[stg].[Post Process Source] NULL, 'sample','Some Source Data', 'Y'
-- =============================================
CREATE PROCEDURE [stg].[Post Process Source] (		@EffectiveDate						DATE = NULL,
													@DestTableSchema					VARCHAR (255) ,
													@DestTableName						VARCHAR (255) ,
													@DestTableUniqueColumnName			VARCHAR (255) ,
													@TableNameWithAllCurrentUniqueKeys	VARCHAR (255) ,  -- table name without a schema. Table schema has to be the same as @DestTableSchema. If defined this table will be used to sync-up the current keys, i.e., expire deleted keys,  during incremental load. This table can be the same as @DestTableName
													@Debug								CHAR (1) = 'N'
												)
AS
BEGIN
-- SET NOCOUNT ON added to prevent extra result sets from interfering with SELECT statements.
	SET NOCOUNT ON

	BEGIN TRY;
		DECLARE @Error				INT
		DECLARE @StepTimestart		DATETIME
		DECLARE @Timestart			DATETIME = GETDATE( )
		DECLARE @ProcessStarted		DATETIME = GETDATE( )
		DECLARE @Message			NVARCHAR(4000);
		DECLARE @Result				INT
		DECLARE @Statement			NVARCHAR(MAX);
		DECLARE @RowsAffected		INT
		DECLARE @NEW_LINE_CHAR		NCHAR(2) = NCHAR(13) + NCHAR(10)

		DECLARE @ErrorMessage		NVARCHAR(4000)
		DECLARE @ErrorSeverity		INT
		DECLARE @ErrorState			INT 

		DECLARE @ExpireUniqueIdsBasedOnUniqueKeysTable		BIT = 0 -- set this to 1 when the table @TableNameWithAllCurrentUniqueKeys is defined to delete keys for current effective date and onward

		DECLARE @PrevEffectiveDate				DATE = NULL
		DECLARE @DestTable						VARCHAR ( 500 )
		DECLARE @DestTableTemp					VARCHAR ( 500 )
		DECLARE @TableWithAllCurrentUniqueKeys	VARCHAR ( 500 )
		DECLARE @TableColumns					VARCHAR(MAX) = ''
		DECLARE @TablecolumnsForRowHash			VARCHAR(MAX) = ''
		DECLARE @MAX_DATA_LENGTH_FOR_HASH		INT = 510 -- This is the max length of the column before using HASHBYTES to compress the column into a hash in order to satisfy max input size of the HASHBYTES for the row

		DECLARE @MaxRowVersion					VARBINARY(8)

		PRINT '*** STARTED Post Process Source at ' + RTRIM( CONVERT(CHAR(40), GETDATE(), 8) ) + @NEW_LINE_CHAR

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 1. Start Post Process Source' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
		END

		SET @Message = '[stg].[Post Process Source] parameters: ' + @NEW_LINE_CHAR
						+ '@EffectiveDate                        - ' + ISNULL(CONVERT( VARCHAR(20), @EffectiveDate, 101 ), 'NULL' ) + @NEW_LINE_CHAR
						+ '@DestTableSchema                      - ' + ISNULL(@DestTableSchema, 'NULL' ) + @NEW_LINE_CHAR
						+ '@DestTableName                        - ' + ISNULL(@DestTableName , 'NULL' ) + @NEW_LINE_CHAR
						+ '@DestTableUniqueColumnName			 - ' + ISNULL(@DestTableUniqueColumnName		, 'NULL' ) + @NEW_LINE_CHAR
						+ '@TableNameWithAllCurrentUniqueKeys	 - ' + ISNULL(@TableNameWithAllCurrentUniqueKeys, 'NULL' ) + @NEW_LINE_CHAR
						+ '@Debug                                - ' + ISNULL(@Debug , 'NULL' ) + @NEW_LINE_CHAR + @NEW_LINE_CHAR
		PRINT @Message


		IF @EffectiveDate IS NULL
			SET @EffectiveDate = GETDATE ()
		
		SET @PrevEffectiveDate = DATEADD( DAY , -1, @EffectiveDate )
		SET @DestTable = QUOTENAME( @DestTableSchema ) + '.' + QUOTENAME( @DestTableName )
		SET @DestTableTemp = QUOTENAME( @DestTableSchema ) + '.' + QUOTENAME( @DestTableName + ' Temp' )


		IF @TableNameWithAllCurrentUniqueKeys IS NOT NULL
		BEGIN
			SET @TableWithAllCurrentUniqueKeys = QUOTENAME( @DestTableSchema ) + '.' + QUOTENAME( @TableNameWithAllCurrentUniqueKeys )

			-- Make sure that this table is not empty. If it is empty use the main table to expire the keys

			DECLARE @KeyTableIsEmpty	BIT = 0

			SELECT @Statement = 
								N'
									IF ( SELECT COUNT(*) FROM ( SELECT TOP 1 1 FROM <@TableWithAllCurrentUniqueKeys> ) AS t ) = 0
										SET @KeyTableIsEmpty = 1
								'
			SELECT @Statement = REPLACE(@Statement, '<@TableWithAllCurrentUniqueKeys>', @TableWithAllCurrentUniqueKeys );

			IF ( @Debug != 'N' )
			BEGIN 
				PRINT 'Point 2. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
				SET @StepTimeStart = GETDATE();

				PRINT 'Processing statement:'
				PRINT CAST( @Statement AS TEXT )
			END

			IF @Statement IS NULL
			BEGIN
				SET @ErrorMessage = 'ERROR! Dynamic SQL statement to determine if key table is empty is NULL. Exiting load.'
				RAISERROR ( @ErrorMessage, 16,0 );
				RETURN 1
			END

			EXECUTE sp_executesql @Statement, N'@KeyTableIsEmpty BIT OUT',
												@KeyTableIsEmpty = @KeyTableIsEmpty OUT

			IF @KeyTableIsEmpty = 1
				SET @TableWithAllCurrentUniqueKeys = @DestTableTemp

			SET @ExpireUniqueIdsBasedOnUniqueKeysTable = 1
		END
		ELSE
		BEGIN	-- Set @TableWithAllCurrentUniqueKeys to @DestTable. It will help compiling the dynamic SQL below but this table will not be used because  @ExpireUniqueIdsBasedOnUniqueKeysTable will be 0
			SET @TableWithAllCurrentUniqueKeys = @DestTable
		END

		PRINT '@TableWithAllCurrentUniqueKeys set to ' + ISNULL( @TableWithAllCurrentUniqueKeys, 'NULL' ) + @NEW_LINE_CHAR


		-- Delete destination table if running for the second time on the first day
		SELECT @Statement = 
							N'
								DECLARE @RowsAffected		INT
						
								IF ( SELECT MIN( [Effective Date Start] ) FROM <@DestTable> ) = ''<@EffectiveDate>''
								BEGIN
									DELETE <@DestTable>

									SELECT @RowsAffected = @@ROWCOUNT
									PRINT ''Deleted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' from <@DestTable> on initial load re-run on the same effective date''
								END
							'
		SELECT @Statement = REPLACE( @Statement, '<@EffectiveDate>', ISNULL( CONVERT( CHAR(10), @EffectiveDate, 120 ), 'NULL' ) );
		SELECT @Statement = REPLACE( @Statement, '<@DestTable>', @DestTable ) ;

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 1a. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();

			PRINT 'Processing deleted statement:'
			PRINT CAST( @Statement AS TEXT )
		END

		EXECUTE sp_executesql @Statement

		CREATE TABLE #Columns
		(
			[Column Number] INT,
			[Column Name]	VARCHAR ( 200),
			[Max Length]	INT,
		)

		-- Create strings that enumerate all columns 
		SELECT @Statement =
				N'
					INSERT #Columns
					SELECT tableColumns.[column_id], tableColumns.[name], IIF( systemTypes.[name] IN ( ''text'' ), 10000, tableColumns.[max_length] )
					FROM sys.columns AS tableColumns
						INNER JOIN sys.types AS systemTypes ON tableColumns.[system_type_id] = systemTypes.[system_type_id]
					WHERE tableColumns.[object_id] = object_id( ''<@DestTableTemp>'' )
						AND tableColumns.[is_identity] = 0
				'
		SELECT @Statement = REPLACE(@Statement, '<@DestTableTemp>', @DestTableTemp );


		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 2. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			PRINT 'Statement to select columns'
			PRINT CAST ( @Statement AS TEXT )
		END
		EXECUTE sp_executesql @Statement

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 3. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			IF ( @Debug != 'P' )
				SELECT '#Columns' AS [#Columns], * 
				FROM #Columns 
		END


	-- Add opening and closing brackets and merge columns again into a 1ist.
		SELECT @TableColumns = COALESCE (@TableColumns + @NEW_LINE_CHAR, '' ) 
							+ ISNULL( QUOTENAME( [Column Name] ) + CASE WHEN [Column Number] != ( SELECT MAX( [Column Number] ) FROM #Columns ) THEN ',' ELSE '' END, '' )
		FROM #Columns
		ORDER BY [Column Number]

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 4. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			PRINT '@TableColumns'
			PRINT CAST ( @TableColumns AS TEXT )
		END

		-- Columns for row hash include all mthe columns except for the Key
		-- Also if the column length is negative ( i.e., length is MAX ) or length exceeds the maximum defined 
		-- here convert the text to hash value so that the total lenth of 
		-- the hashed string does not exceed the SQL Server maximum of 8000 chracters 
		SELECT @TableColumnsForRowHash = COALESCE (@TableColumnsForRowHash + @NEW_LINE_CHAR, '' ) 
							+ ISNULL( CASE WHEN [Max Length] < 0 OR [Max Length] > @MAX_DATA_LENGTH_FOR_HASH
											THEN 'CONVERT(VARCHAR(32), HASHBYTES( ''SHA_256'', SUBSTRING( CAST (' + QUOTENAME( [Column Name] ) + ' AS VARCHAR( 8000 ) ), 1, 8000 ) ), 2 ) AS ' + QUOTENAME( [Column Name] )
											ELSE QUOTENAME( [Column Name] ) END 
									+ CASE WHEN [Column Number] != ( SELECT MAX( [Column Number] ) FROM #Columns WHERE [Column Number] NOT IN ( @DestTableUniqueColumnName, 'Created On' ) ) THEN ',' ELSE '' END, '' )
		FROM #Columns
		WHERE [Column Name] NOT IN ( @DestTableUniqueColumnName, 'Created On' )  -- TODO. 
		ORDER BY [Column Number]


		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 4. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			PRINT 'Table columns after brackets are added'
			PRINT CAST ( @TableColumns AS TEXT )

			PRINT 'Columns for row hash after brackets are added'
			PRINT CAST ( @TableColumnsForRowHash AS TEXT )
		END


		--- Load source data
		SELECT @Statement =
				N'
					DECLARE @ErrorMessage		NVARCHAR(4000)
					DECLARE @StepTimestart		DATETIME = GETDATE( )
					DECLARE @RowsAffected		INT
					DECLARE @InitialLoad		BIT = 0
					DECLARE @NEW_LINE_CHAR		NCHAR(2) = NCHAR(13) + NCHAR(10)
					DECLARE @FAR_AWAY_DATE		DATE = ''20990101''

					SELECT
							CAST( ( SELECT HASHBYTES( ''SHA2_256'', ( SELECT	<@TableColumnsForRowHash> FOR XML RAW ) ) ) AS BINARY (32) )	AS [Row Hash],
							<@TableColumns>
					INTO #NewData
					FROM <@DestTableTemp>

					SELECT @RowsAffected = @@ROWCOUNT
					PRINT ''Detected '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' new rows in <@DestTable> feed for <@EffectiveDate> effective date ''
					CREATE INDEX [#NewData Indx1] ON #NewData ( [<@DestTableUniqueColumnName>], [Row Hash] )


					IF ( ''<@Debug>'' != ''N'' )
					BEGIN 
						PRINT ''Point 5. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
						SET @StepTimeStart = GETDATE();
					END

					IF NOT EXIST ( SELECT 1 FROM <@DestTable> )
						SET @InitialLoad = 1 

					BEGIN TRAN

					-- 1. This step is only needed on re-runs when the records were removed from the feed or changed
					--		Delete records where [Effective Date Start] is equal to @EffectiveDate
					IF @InitialLoad = 0
					BEGIN

						IF <@ExpireUniqueIdsBasedOnUniqueKeysTable> = 1
						BEGIN
								-- Restore expired versions that do not have the latest version, i.e., deleted as of today, and do not have the latest version or changed columns
							;WITH StgTableWithVersionNumber
							AS
							(
								SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
															ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
										*
								FROM <@DestTable>
							)
							UPDATE stg
							SET		[Effective Date End] = @FAR_AWAY_DATE
							FROM StgTableWithVersionNumber AS stg
								INNER JOIN <@TableWithAllCurrentUniqueKeys> AS new ON stg.[<@DestTableUniqueColumnName>] = new.[<@DestTableUniqueColumnName>]
							WHERE	stg.[Reverse Version Number] = 1
								AND stg.[Effective Date End] = ''<@PrevEffectiveDate>''
										
							SELECT @RowsAffected = @@ROWCOUNT
							PRINT ''Restored '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows in <@DestTable> for <@EffectiveDate> effective date for Ids that are in the feed and whose [Effective Date End] does not equal to @FAR_AWAY_DATE''

								-- Delete versions that are no longer in the feed
							;WITH StgTableWithVersionNumber
							AS
							(
								SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
															ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
										*
								FROM <@DestTable>
							)
							DELETE stg
							FROM StgTableWithVersionNumber AS stg
							WHERE NOT EXISTS ( 	SELECT 1
												FROM <@TableWithAllCurrentUniqueKeys> AS new 
												WHERE stg.[<@DestTableUniqueColumnName>] = new.[<@DestTableUniqueColumnName>] )
								AND stg.[Reverse Version Number] = 1
								AND stg.[Effective Date End] = ''<EffectiveDate>''
										
							SELECT @RowsAffected = @@ROWCOUNT
							PRINT ''Deleted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows in <@DestTable> for <@EffectiveDate> effective date for Ids that are no longer in the feed''
						END

						-- For both cases, i.e., when we have a table with keys or not, delete rows that do not exist or chaged. It will go over the same rows as previous delete
						-- when the table with key is present, but will also delete rows that are present but changed
						;WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
							*
							FROM <@DestTable>
						)
						DELETE stg
						FROM StgTableWithVersionNumber AS stg
						WHERE NOT EXISTS (	SELECT 1
											FROM #NewData AS new
											WHERE		stg.[<@DestTableUniqueColumnName>]	= new.[<@DestTableUniqueColumnName>]
													AND stg.[Row Hash] = new.[Row Hash] )
							AND stg.[Reverse Version Number] = 1
							AND stg.[Effective Date Start] = ''<@EffectiveDate>''

						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Deleted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows in <@DestTable> for <@EffectiveDate> effective date for Ids that are no longer in the feed or has changed''

						IF ( ''<@Debug>'' != ''N'' )
						BEGIN 
							PRINT ''Point 6. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
							SET @StepTimeStart = GETDATE();
						END
					END


					-- 2. Set [Effective Date End] to the prior date on the most recent record that has [Effective Date End] set to @FAR_AWAY_DATE
					--		if it was deleted or changed for tabled not loaded incrementally
					--		or changed for tables loaded incrementally
					IF @InitialLoad = 0
					BEGIN

						IF <@ExpireUniqueIdsBasedOnUniqueKeysTable> = 1
						BEGIN
							;WITH StgTableWithVersionNumber
							AS
							(
								SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
															ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
								*
								FROM <@DestTable>
							)
							UPDATE stg
							SET		[Effective Date End] = ''<@PrevEffectiveDate>''
							FROM StgTableWithVersionNumber AS stg
							WHERE NOT EXISTS (	SELECT 1
												FROM <@TableWithAllCurrentUniqueKeys> AS allKeys
												WHERE		stg.[<@DestTableUniqueColumnName>]		= allKeys.[<@DestTableUniqueColumnName>] 
											  )
								AND stg.[Reverse Version Number] = 1
								AND stg.[Effective Date End] = @FAR_AWAY_DATE

							SELECT @RowsAffected = @@ROWCOUNT
							PRINT ''Updated '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows in <@DestTable> with previous effective date for <@EffectiveDate> effective date for Ids that are no longer in the feed or has changed''
						END
						
						-- in addition expire all changed rows
						;WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
							*
							FROM <@DestTable>
						)
						UPDATE stg
						SET		[Effective Date End] = ''<@PrevEffectiveDate>''
						FROM StgTableWithVersionNumber AS stg
							INNER JOIN #NewData AS new ON stg.[<@DestTableUniqueColumnName>] = new.[<@DestTableUniqueColumnName>]
						WHERE	stg.[Reverse Version Number] = 1
							AND stg.[Effective Date End] = @FAR_AWAY_DATE
							AND stg.[Row Hash] != new.[Row Hash]
										
						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Updated '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows in <@DestTable> with previous effective date for <@EffectiveDate> effective date for Ids that were changed''

						IF ( ''<@Debug>'' != ''N'' )
						BEGIN 
							PRINT ''Point 7. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
							SET @StepTimeStart = GETDATE();
						END
					END

					-- 3. Insert records for new Id or new version for the existing id
					--		On re-run the number of rows here can be zero
					IF @InitialLoad = 1
					BEGIN
						INSERT <@DestTable>
						(
							[Effective Date Start]	,
							[Effective Date End]	,
							[Row Hash]				,
							<@TableColumns>
						)
						SELECT -- Set Effective date
							''<@EffectiveDate>''		AS [Effective Date Start]	,
							@FAR_AWAY_DATE				AS [Effective Date End]		,
							[Row Hash]					AS [Row Hash]				,
							<@TableColumns>
						FROM #NewData AS new

						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Inserted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows with new key or new version in <@DestTable> for <@EffectiveDate> effective date on initial load''
					END
					ELSE 
					BEGIN
						;WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
							*
							FROM <@DestTable>
						)
						INSERT <@DestTable>
						(
							[Effective Date Start]	,
							[Effective Date End]	,
							[Row Hash]				,
							<@TableColumns>
						)
						SELECT -- Set Effective date
							''<@EffectiveDate>''		AS [Effective Date Start]	,
							@FAR_AWAY_DATE				AS [Effective Date End]		,
							[Row Hash]					AS [Row Hash]				,
							<@TableColumns>
						FROM #NewData AS new
						WHERE NOT EXISTS (	SELECT 1
											FROM StgTableWithVersionNumber AS stg
											WHERE		stg.[<@DestTableUniqueColumnName>]		= new. [<@DestTableUniqueColumnName>]
													AND stg.[Row Hash]	= new.[Row Hash]
													AND stg.[Reverse Version Number] = 1 )

						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Inserted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows with new key or new version in <@DestTable> for <@EffectiveDate> effective date''

						-- Insert new version after delete when the row hash is the same
						;WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
							*
							FROM <@DestTable>
						)
						INSERT <@DestTable>
						(
							[Effective Date Start]	,
							[Effective Date End]	,
							[Row Hash]				,
							<@TableColumns>
						)
						SELECT -- Set Effective date
							''<@EffectiveDate>''		AS [Effective Date Start]	,
							@FAR_AWAY_DATE				AS [Effective Date End]		,
							[Row Hash]					AS [Row Hash]				,
							<@TableColumns>
						FROM #NewData AS new
						WHERE EXISTS	(	SELECT 1
											FROM StgTableWithVersionNumber AS stg
											WHERE		stg.[<@DestTableUniqueColumnName>]	= new. [<@DestTableUniqueColumnName>]
													AND stg.[Row Hash]						= new.[Row Hash]
													AND stg.[Effective Date End]			< ''<@PrevEffectiveDate>''
													AND stg.[Reverse Version Number] = 1 )

						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Inserted '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows that re-appeared after the delete in <@DestTable> feed for <@EffectiveDate> effective date''
					END

					IF ( ''<@Debug>'' != ''N'' )
					BEGIN 
						PRINT ''Point 8. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
						SET @StepTimeStart = GETDATE();
					END

					-- 4. Set version on newly inserted rows
					IF @InitialLoad = 0
					BEGIN
						; WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] ASC ) AS [Version Number],
							[Version]
							FROM <@DestTable>
						)
						UPDATE StgTableWithVersionNumber
						SET [Version] = [Version Number]
						WHERE [Version] = 0					-- only need to update new rows. New rows will have version number set to 0 - the default
					
						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Updated '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows with [Version] column in <@DestTable> for <@EffectiveDate> effective date''

						IF ( ''<@Debug>'' != ''N'' )
						BEGIN 
							PRINT ''Point 9. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
							SET @StepTimeStart = GETDATE();
						END
					END

					-- 5. Update the [Effective Date End] for all previous versions that are out of sync
					IF @InitialLoad = 0
					BEGIN
						;WITH StgTableWithVersionNumber
						AS
						(
							SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
														ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
							*
							FROM <@DestTable>
						)
						UPDATE prevVer
						SET [Effective Date End] = DATEADD( DAY, -1, lastVer.[Effective Date Start] )
						FROM StgTableWithVersionNumber AS lastVer
							INNER JOIN StgTableWithVersionNumber AS prevVer ON		lastVer.[<@DestTableUniqueColumnName>] = prevVer.[<@DestTableUniqueColumnName>]
																				AND	lastVer.[Reverse Version Number] = 1
																				AND prevVer.[Reverse Version Number] = 2
						WHERE prevVer.[Effective Date End] > DATEADD( DAY, -1, lastVer.[Effective Date Start] )
					
						SELECT @RowsAffected = @@ROWCOUNT
						PRINT ''Updated '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows to set [Effective Date End] on prior to last version in <@DestTable> for <@EffectiveDate> effective date''
						
						IF ( ''<@Debug>'' != ''N'' )
						BEGIN 
							PRINT ''Point 11. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
							SET @StepTimeStart = GETDATE();
						END
					END



					--7. This step is no longer needed.
					--   This will ahppen automatically
					--		Move forward the [Effective Date End] for all Ids that are not present in the feed
					--		This step is only applicable for feeds that do not phisically delete records 
					--		For feeds that delete records this step cannot be used
					--;WITH StgTableWithVersionNumber
					--AS
					--(
					--	SELECT ROW_NUMBER() OVER (	PARTITION BY [<@DestTableUniqueColumnName>]
					--								ORDER BY [Effective Date Start] DESC ) AS [Reverse Version Number] ,
					--	*
					--	FROM <@DestTable>
					--)
					--UPDATE StgTableWithVersionNumber
					--SET [Effective Date End] = ''<@EffectiveDate>''
					--WHERE	[Reverse Version Number] = 1
					--	AND [Effective Date End] != ''<@EffectiveDate>''

					--SELECT @RowsAffected = @@ROWCOUNT
					--PRINT ''Updated '' + CONVERT( VARCHAR(12), @RowsAffected ) + '' rows to move forward [Effective Date End] for Ids not present in the feed in <@DestTable> for <@EffectiveDate> effective date''

					--IF ( ''<@Debug>'' != ''N'' )
					--BEGIN 
					--	PRINT ''Point 10. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
					--	SET @StepTimeStart = GETDATE();
					--END


					COMMIT TRAN


					--- Verify data. The check is done after the data is committed to simplify error detection
					-- The load would need to be corrected and re-run in case of an error
					
					-- 1. Verify that there are no rows with zero version
					IF EXISTS ( SELECT 1 FROM <@DestTable> WHERE [Version] = 0 )
					BEGIN
						SET @ErrorMessage = ''ERROR! Table <@DestTable> has some rows with zero version. Fix the issue and re-run the load''
						RAISERROR ( @ErrorMessage, 16,0 );
					END

					IF ( ''<@Debug>'' != ''N'' )
					BEGIN 
						PRINT ''Point 13. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
						SET @StepTimeStart = GETDATE();
					END

					-- 2. Verify that versions are continuous
					IF EXISTS ( SELECT 1 
								FROM <@DestTable> 
								GROUP BY [<@DestTableUniqueColumnName>]
								HAVING MAX( [Version] ) != COUNT(*)
							)
					BEGIN
						SET @ErrorMessage = ''ERROR! Table <@DestTable> has mismatched row versions. Fix the issue and re-run the load''
						RAISERROR ( @ErrorMessage, 16,0 );
					END

					IF ( ''<@Debug>'' != ''N'' )
					BEGIN 
						PRINT ''Point 14. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
						SET @StepTimeStart = GETDATE();
					END


					-- 3. Verify that Start and End dates are not overlapping
					--		They do not have to be continuous as records can re-appar after being deleted
					-- The SQL Server was taking foreever wehn EXISTS was used in the query
					SELECT 1 AS [Row]
					INTO #CheckForNotOverlapping
					FROM <@DestTable> AS currVer
						INNER JOIN <@DestTable> AS nextVer ON		currVer.[<@DestTableUniqueColumnName>] = nextVer.[<@DestTableUniqueColumnName>]
																AND currVer.[Version] = nextVer.[Version] - 1
					WHERE currver. [Effective Date End] >= nextVer.[Effective Date Start]


					IF EXISTS ( SELECT 1
								FROM #CheckForNotOverlapping )
					BEGIN
						SET @ErrorMessage = ''ERROR! Table <@DestTable> has mismatched Start and End Effective Dates. Fix the issue and re-run the load''
						RAISERROR ( @ErrorMessage, 16,0 );
					END

					IF ( ''<@Debug>'' != ''N'' )
					BEGIN 
						PRINT ''Point 14. '' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + '' sec  '' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
						SET @StepTimeStart = GETDATE();
					END

					PRINT ''Data in the <@DestTable> has been verified for consistency''

					-- 4. Extract max row version
					SELECT @MaxRowVersion = MAX( [Row Version] )
					FROM <@DestTable>
				'

		SELECT @Statement = REPLACE( @Statement, '<@EffectiveDate>', ISNULL( CONVERT( CHAR(10), @EffectiveDate, 120 ), 'NULL' ) );
		SELECT @Statement = REPLACE( @Statement, '<@PrevEffectiveDate>', ISNULL( CONVERT( CHAR(10), @PrevEffectiveDate, 120 ), 'NULL') );
		SELECT @Statement = REPLACE( @Statement, '<@DestTable>', @DestTable ) ;
		SELECT @Statement = REPLACE( @Statement, '<@DestTableTemp>', @DestTableTemp );
		SELECT @Statement = REPLACE( @Statement, '<@TableWithAllCurrentUniqueKeys>', @TableWithAllCurrentUniqueKeys );
		SELECT @Statement = REPLACE( @Statement, '<@TableColumns>', @TableColumns );
		SELECT @statement = REPLACE( @statement, '<@TableColumnsForRowHash>', IIF( @TableColumnsForRowHash = '', '''''', @TableColumnsForRowHash + ' FOR XML RAW ' ) ) ;  -- if there are no columns for hash, i.e., all fields are a key  pass two quotes
		SELECT @Statement = REPLACE( @Statement, '<@DestTableUniqueColumnName>', @DestTableUniqueColumnName );
		SELECT @Statement = REPLACE( @Statement, '<@ExpireUniqueIdsBasedOnUniqueKeysTable>', CAST( @ExpireUniqueIdsBasedOnUniqueKeysTable AS VARCHAR(10) ) );
		SELECT @Statement = REPLACE( @Statement, '<@Debug>', @Debug )

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point 4. ' + CONVERT(VARCHAR(6),DATEDIFF(ss,@StepTimeStart, GETDATE())) + ' sec  ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8));
			SET @StepTimeStart = GETDATE();
			
			PRINT 'Processing Statement: '
			PRINT CAST ( @Statement AS TEXT )
		END


		IF @Statement IS NULL
		BEGIN
			RAISERROR ( 'ERROR RUNNING [stg].[Post Process Source]! Dynamically built SQL is NULL. Load failed.',  16, 0 );
			RETURN 1
		END

		EXECUTE sp_executesql @Statement, ' @MaxRowVersion  VARBINARY(8)	OUT',
											@MaxRowVersion = @MaxRowVersion OUT

		INSERT [stg].[Load Control] ( [Effective Date] , [Table Processed] , [Description] , [Load Duration Sec], [Result] , [Error Message], [Max Row Version] )
		VALUES ( @EffectiveDate, @DestTable, NULL, CAST( DATEDIFF( MILLISECOND, @TimeStart, GETDATE() ) AS FLOAT ) / 1000., 'Success', NULL, @MaxRowVersion )

		IF ( @Debug != 'N' )
		BEGIN 
			PRINT 'Point last. ' + CONVERT( CHAR(6), DATEDIFF( ss, @StepTimeStart, GETDATE() ) ) + 'sec ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8))
		END	

		PRINT '*** COMPLETED Post Process Source in ' + CONVERT( CHAR(6), DATEDIFF( ss, @TimeStart, GETDATE() ) ) + ' sec at ' + RTRIM(CONVERT(CHAR(40), GETDATE(), 8)) + @NEW_LINE_CHAR
		
		RETURN 0

	END TRY
	BEGIN CATCH
		SELECT	@ErrorMessage  =
					  '   ERROR_NUMBER: ' + ISNULL( CONVERT(VARCHAR(6),ERROR_NUMBER()),'')  + CHAR(13)
					+ '   ERROR_SEVERITY: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_SEVERITY()),'')  + CHAR(13)
					+ '   ERROR_STATE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_STATE()),'')  + CHAR(13)
					+ '   ERROR_PROCEDURE: '  + ISNULL( ERROR_PROCEDURE(),'')  + CHAR(13)
					+ '   ERROR_LINE: '  + ISNULL( CONVERT(VARCHAR(6),ERROR_LINE()),'') + CHAR(13)
					+ '   ERROR_MESSAGE: '  + ISNULL( ERROR_MESSAGE(),'')  + CHAR(13),
				@ErrorSeverity = ERROR_SEVERITY(),
				@ErrorState = ERROR_STATE();

		INSERT [api].[Load Control] ( [Effective Date] , [Table Processed] , [Description] , [Load Duration Sec], [Result] , [Error Message] )
		VALUES ( @EffectiveDate, @DestTable, NULL, CAST( DATEDIFF( MILLISECOND, @TimeStart, GETDATE() ) AS FLOAT ) / 1000., 'Error', @ErrorMessage )
	
		IF @@TRANCOUNT > 0
			ROLLBACK TRANSACTION

	    RAISERROR ( @ErrorMessage, @ErrorSeverity, @ErrorState );

		RETURN 1
	END CATCH

END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stg].[Post Process Source]') AND type IN (N'P', N'PC'))
BEGIN
	PRINT 'CREATED PROCEDURE [stg].[Post Process Source]'
END
GO
