﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Globalization;

namespace DWJobs.Common.AppEnvironment
{
    /// <summary>
    ///  Configuration
    /// </summary>
    public partial class AppConfiguration
    {

        public enum JobType
        {
            NotSpecified,
            LoadDims,
            LoadFacts,
            LoadDimsFacts,
            LoadSsas,
            LoadFactsSsas,
            LoadDimsFactsSsas,

            UpdateStatisticsInDW,
            CleanUpAllSessionsInDW,
            CleanUpAllSessionsInDimAuth,

            LoadAdj,
            LoadAdjSsas,
            LoadSsasAdj,
            LoadSsasAdjSsas,
            LoadFactsSsasAdj,
            LoadFactsSsasAdjSsas,
            LoadDimsFactsSsasAdj,
            LoadDimsFactsSsasAdjSsas,
            LoadDimsFactsAdjSsas,
            LoadDimsFactsAdj,
            LoadFactsAdjSsas,
            LoadFactsAdj,
            MaintenanceFacts,

            LoadSourceData,
            ExecuteXMLA,
            ExecuteMDX,
            UpdateStatisticsInStagingDB,
        } ;
        
        
        
        public static string MailToList
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().MailToList;
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().MailToList = value;
            }
        }

        public static string DimServerName
        {
            get
            {
                return (string) AppConfiguration.GetEnvironmentSettings().DimServerName;
            }
        }


        public static string DimDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DimDBName;
            }
        }
        
        public static string SourceServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceServerName;
            }
        }

        public static string SourceDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceDBName;
            }
        }

        public static string DimAuthorityServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DimAuthorityServerName;
            }
        }

        public static string DimAuthorityDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DimAuthorityDBName;
            }
        }

        public static int NumberOfParallelProcesses
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().NumberOfParallelProcesses))
                        return 0;

                    return Int32.Parse(AppConfiguration.GetEnvironmentSettings().NumberOfParallelProcesses);
                }
                catch
                {
                    return 0;   // Default is zero run without any restrictions
                }
            }
        }

        public static int SSASGoBackDaysForStartDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().SSASGoBackDaysForStartDateToLoad))
                        return 0;

                    return Int32.Parse(AppConfiguration.GetEnvironmentSettings().SSASGoBackDaysForStartDateToLoad);
                }
                catch
                {
                    return 0;   // Default is zero to load all days
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASGoBackDaysForStartDateToLoad = value.ToString();
            }

        }

        public static int SSASMaxParallel
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().SSASMaxParallel))
                        return 0;

                    return Int32.Parse(AppConfiguration.GetEnvironmentSettings().SSASMaxParallel);
                }
                catch
                {
                    return 0;
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASMaxParallel = value.ToString();
            }
        }

        public static int TimeoutMinutesOnParallelProcesses
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().TimeoutMinutesOnParallelProcesses))
                        return 0;

                    return Int32.Parse(AppConfiguration.GetEnvironmentSettings().TimeoutMinutesOnParallelProcesses);
                }
                catch
                {
                    return 0;
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().TimeoutMinutesOnParallelProcesses = value.ToString();
            }
        }

        public static double FloatCompareTheshold
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().FloatCompareTheshold))
                        return 0;

                    return Double.Parse(AppConfiguration.GetEnvironmentSettings().FloatCompareTheshold);
                }
                catch
                {
                    return 0;
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().FloatCompareTheshold = value.ToString();
            }
        }

        public static int GoBackDaysForStartDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().GoBackDaysForStartDateToLoad))
                        return 0;

                    return Int32.Parse(AppConfiguration.GetEnvironmentSettings().GoBackDaysForStartDateToLoad);
                }
                catch
                {
                    return 0;
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().GoBackDaysForStartDateToLoad = value.ToString();
            }
        }

        public static bool IsInitialLoad
        {
            get
            {
                return (bool)(AppConfiguration.GetEnvironmentSettings().GetConfigurationProperty("initialLoad") != "Y" ? false : true);
            }
        }

        public static bool IsNonInitialFullLoad
        {
            get
            {
                return (bool)(AppConfiguration.GetEnvironmentSettings().IsNonInitialFullLoad != "Y" ? false : true);
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().IsNonInitialFullLoad = (value ? "Y" : "N");
            }
        }

        public static DateTime? StartDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().StartDateToLoad))
                        return null;

                    CultureInfo provider = CultureInfo.InvariantCulture;
                    return DateTime.ParseExact(AppConfiguration.GetEnvironmentSettings().StartDateToLoad, "d", provider);
                }
                catch
                {
                    AppConfiguration.Log.ErrorFormat("StartDateToLoad must be in MM/DD/YYYY format. The value is {0} ", AppConfiguration.GetEnvironmentSettings().StartDateToLoad);
                    throw new ApplicationException("Error getting StartDateToLoad");
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().StartDateToLoad = value.Value.ToString("MM/dd/yyyy" );
            }
        }

        public static DateTime? SSASStartDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().SSASStartDateToLoad))
                        return null;

                    CultureInfo provider = CultureInfo.InvariantCulture;
                    return DateTime.ParseExact(AppConfiguration.GetEnvironmentSettings().SSASStartDateToLoad, "d", provider);
                }
                catch
                {
                    AppConfiguration.Log.ErrorFormat("SSASStartDateToLoad must be in MM/DD/YYYY format. The value is {0} ", AppConfiguration.GetEnvironmentSettings().SSASStartDateToLoad);
                    throw new ApplicationException("Error getting SSASStartDateToLoad");
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASStartDateToLoad = value.Value.ToString("MM/dd/yyyy" );
            }
        }

        public static DateTime? DIMStartDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().DIMStartDateToLoad))
                        return null;

                    CultureInfo provider = CultureInfo.InvariantCulture;
                    return DateTime.ParseExact(AppConfiguration.GetEnvironmentSettings().DIMStartDateToLoad, "d", provider);
                }
                catch
                {
                    AppConfiguration.Log.ErrorFormat("DIMStartDateToLoad must be in MM/DD/YYYY format. The value is {0} ", AppConfiguration.GetEnvironmentSettings().DIMStartDateToLoad);
                    throw new ApplicationException("Error getting DIMStartDateToLoad");
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASStartDateToLoad = value.Value.ToString("MM/dd/yyyy" );
            }
        }

        public static DateTime? EndDateToLoad
        {
            get
            {
                try
                {
                    if (String.IsNullOrEmpty(AppConfiguration.GetEnvironmentSettings().EndDateToLoad))
                        return null;

                    CultureInfo provider = CultureInfo.InvariantCulture;
                    return DateTime.ParseExact(AppConfiguration.GetEnvironmentSettings().EndDateToLoad, "d", provider);
                }
                catch
                {
                    AppConfiguration.Log.ErrorFormat("EndDateToLoad must be in MM/DD/YYYY format. The value is {0} ", AppConfiguration.GetEnvironmentSettings().EndDateToLoad);
                    throw new ApplicationException("Error getting EndDateToLoad");
                }
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().EndDateToLoad = value.Value.ToString("MM/dd/yyyy" );
            }
        }

        public static string DimensionAuthorityDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DimensionAuthorityDBConnectionString;
            }
        }

        public static string ReportingDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().ReportingDBConnectionString;
            }
        }

        public static string DestDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DestDBConnectionString;
            }
        }

        public static string Debug
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().Debug;
            }
        }

        public static string DWDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DWDBConnectionString;
            }
        }

        public static string SSASSourceDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SSASSourceDBConnectionString;
            }
        }

        public static string SSASDestDBConnectionString
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().ssasDestDBConnectionString;
            }
        }

        public static string DBProviderName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DBProviderName;
            }
        }

        public static string SSASServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SSASServerName;
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASServerName = value;
            }
        }

        public static string SSASProcessingServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SSASProcessingServerName;
            }
        }

        public static string SourceSSASServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceSSASServerName;
            }
        }

        public static string TargetSSASServerName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().TargetSSASServerName;
            }
        }

        public static string SSASDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SSASDBName;
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().SSASDBName = value;
            }
        }

        public static string SourceSSASDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceSSASDBName;
            }
        }

        public static string TargetSSASDBName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().TargetSSASDBName;
            }
        }

         public static string SourceSSASCubeName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceSSASCubeName;
            }
        }

        public static string TargetSSASCubeName
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().TargetSSASCubeName;
            }
        }

       public static bool IsSSASFullLoad
        {
            get
            {
                return (bool)(AppConfiguration.GetEnvironmentSettings().IsSSASFullLoad != "Y" ? false : true);
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().IsSSASFullLoad = (value ? "Y" : "N");
            }
        }

       public static bool IsSSASLoadCalculations
        {
            get
            {
                return (bool)(AppConfiguration.GetEnvironmentSettings().GetConfigurationProperty("ssasLoadCalculations") != "Y" ? false : true);
            }
        }

        public static string ProcessedFilesDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().ProcessedFilesDir;
            }
        }

        public static string OutputDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().OutputDir;
            }
        }

        public static string SourceFilesDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().SourceFilesDir;
            }
        }

        public static string InputFile
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().InputFile;
            }
        }

        public static string InputConfigFile
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().InputConfigFile;
            }
        }

        public static string DbImportFilesDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DbImportFilesDir;
            }
        }

        public static string DbImportFilesShareOrExternalDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().DbImportFilesShareOrExternalDir;
            }
        }

        public static JobType Job
        {
            get
            {
                string jobType = (String.IsNullOrEmpty(AppConfiguration.jobExplicit) ? AppConfiguration.GetEnvironmentSettings().JobType : AppConfiguration.jobExplicit);
                foreach (string job in Enum.GetNames(typeof(JobType)))
                    if (String.Compare(job, jobType, true) == 0)
                        return (JobType)Enum.Parse(typeof(JobType), jobType, true);

                return JobType.NotSpecified;
            }
            set
            {
                AppConfiguration.GetEnvironmentSettings().JobType = value.ToString();
            }
        }

        public static string CubeXmla
        {
            get
            {
                return (string) AppConfiguration.GetEnvironmentSettings().CubeXmla;
            }
        }

        public static bool IsFactAdjustmentsLoad
        {
            get
            {
                return (bool)(AppConfiguration.GetEnvironmentSettings().FactAdjustmentsLoad != "Y" ? false : true);
            }
        }

        public static int  MaxDatesToLoad
        {
            get
            {
                int nValue = -1;
                if( Int32.TryParse( (string) AppConfiguration.GetEnvironmentSettings().MaxDatesToLoad, out nValue ) )
                    return nValue;
                else
                    return -1;
            }
        }

        /// <summary>
        /// Path to XML configuration file containing cube permissions.  The schema 
        /// shall conform to schema as defined in CubePermissionsSchema.xsd
        /// </summary>
        public static string CubePermissionsConfiguration
        {
            get
            {
                return (string) AppConfiguration.GetEnvironmentSettings().CubePermissionsConfiguration;
            }
        }
   
    }
}
