﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System.Configuration;

namespace DWJobs.Common.AppEnvironment
{
    internal class AppEnvironmentSettingsCollection : ConfigurationElementCollection 
    {
        public override ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.AddRemoveClearMap;
            }
        }

        public AppEnvironmentSettingsElement this[int index]
        {
            get
            {
                return (AppEnvironmentSettingsElement)BaseGet(index);
            }
            set
            {
                if (BaseGet(index) != null)
                    BaseRemoveAt(index);
                BaseAdd(index, value);
            }
        }

        public new AppEnvironmentSettingsElement this[string key]
        {
            get
            {
                for (int index = 0; index < Count; index++)
                {
                    if (this[index].Name == key)
                        return (AppEnvironmentSettingsElement)BaseGet(index);
                }
                return null;
            }
        }

        public void Add(AppEnvironmentSettingsElement element)
        {
            BaseAdd(element);
        }

        public void Clear()
        {
            BaseClear();
        }

        protected override ConfigurationElement CreateNewElement()
        {
            return new AppEnvironmentSettingsElement();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((AppEnvironmentSettingsElement)element).Name;
        }

        public void Remove(AppEnvironmentSettingsElement element)
        {
            BaseRemove(element.Name);
        }

        public void Remove(string name)
        {
            BaseRemove(name);
        }

        public void RemoveAt(int index)
        {
            BaseRemoveAt(index);
        }
    }
}
