﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Configuration;
using System.Collections.Generic;

namespace DWJobs.Common.AppEnvironment
{
    internal class AppEnvironmentSettingsElement : ConfigurationElement
    {

        private Dictionary<string, string> explicitValue = new Dictionary<string, string>();


        [ConfigurationProperty("name", IsRequired = true, DefaultValue = "No description.")]
        public string Name
        {
            get
            {
                return (string)this["name"];
            }
        }

        [ConfigurationProperty("jobType", IsRequired = true, DefaultValue = "Error. No jobType specified.")]
        public string JobType
        {
            get
            {
                return (explicitValue.ContainsKey("jobType") ? explicitValue["jobType"] : (string)this["jobType"]);
            }
            set
            {
                explicitValue["jobType"] = value;
            }
        }

        [ConfigurationProperty("description", IsRequired = true, DefaultValue = "No description.")]
        public string Description
        {
            get
            {
                return (string)this["description"];
            }
        }

        [ConfigurationProperty("mailToList", IsRequired = false, DefaultValue = "")]
        public string MailToList
        {
            get
            {
                return ( explicitValue.ContainsKey("mailToList") ? explicitValue["mailToList"] : (string)this["mailToList"] );
            }
            set
            {
                explicitValue["mailToList"] = value;
            }
        }

        [ConfigurationProperty("logFileDir", IsRequired = false, DefaultValue = "No description.")]
        public string LogFileDir
        {
            get
            {
                return (string)this["logFileDir"];
            }
        }

        [ConfigurationProperty("dimServerName", IsRequired = false, DefaultValue = "")]
        public string DimServerName
        {
            get
            {
                return (string)this["dimServerName"];
            }
        }

        [ConfigurationProperty("dimDBName", IsRequired = false, DefaultValue = " Dimension Authority Prod")]
        public string DimDBName
        {
            get
            {
                return (string)this["dimDBName"];
            }
        }

        [ConfigurationProperty("sourceServerName", IsRequired = false, DefaultValue = "")]
        public string SourceServerName
        {
            get
            {
                return (string)this["sourceServerName"];
            }
        }

        [ConfigurationProperty("sourceDBName", IsRequired = false, DefaultValue = "")]
        public string SourceDBName
        {
            get
            {
                return (string)this["sourceDBName"];
            }
        }

        [ConfigurationProperty("dimAuthorityServerName", IsRequired = false, DefaultValue = "")]
        public string DimAuthorityServerName
        {
            get
            {
                return (string)this["dimAuthorityServerName"];
            }
        }

        [ConfigurationProperty("dimAuthorityDBName", IsRequired = false, DefaultValue = "")]
        public string DimAuthorityDBName
        {
            get
            {
                return (string)this["dimAuthorityDBName"];
            }
        }

        [ConfigurationProperty("numberOfParallelProcesses", IsRequired = false, DefaultValue = "")]
        public string NumberOfParallelProcesses
        {
            get
            {
                return ( explicitValue.ContainsKey("numberOfParallelProcesses") ? explicitValue["numberOfParallelProcesses"] : (string)this["numberOfParallelProcesses"] );
            }
            set
            {
                explicitValue["numberOfParallelProcesses"] = value;
            }
        }

        [ConfigurationProperty("ssasGoBackDaysForStartDateToLoad", IsRequired = false, DefaultValue = "0")]
        public string SSASGoBackDaysForStartDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("ssasGoBackDaysForStartDateToLoad") ? explicitValue["ssasGoBackDaysForStartDateToLoad"] : (string)this["ssasGoBackDaysForStartDateToLoad"] );
            }
            set
            {
                explicitValue["ssasGoBackDaysForStartDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("ssasMaxParallel", IsRequired = false, DefaultValue = "0")]
        public string SSASMaxParallel
        {
            get
            {
                return ( explicitValue.ContainsKey("ssasMaxParallel") ? explicitValue["ssasMaxParallel"] : (string)this["ssasMaxParallel"] );
            }
            set
            {
                explicitValue["ssasMaxParallel"] = value;
            }
        }

        [ConfigurationProperty("timeoutMinutesOnParallelProcesses", IsRequired = false, DefaultValue = "60")]
        public string TimeoutMinutesOnParallelProcesses
        {
            get
            {
                return ( explicitValue.ContainsKey("timeoutMinutesOnParallelProcesses") ? explicitValue["timeoutMinutesOnParallelProcesses"] : (string)this["timeoutMinutesOnParallelProcesses"] );
            }
            set
            {
                explicitValue["timeoutMinutesOnParallelProcesses"] = value;
            }
        }

        [ConfigurationProperty("floatCompareTheshold", IsRequired = false, DefaultValue = "0.01")]
        public string FloatCompareTheshold
        {
            get
            {
                return ( explicitValue.ContainsKey("floatCompareTheshold") ? explicitValue["floatCompareTheshold"] : (string)this["floatCompareTheshold"] );
            }
            set
            {
                explicitValue["floatCompareTheshold"] = value;
            }
        }

        [ConfigurationProperty("goBackDaysForStartDateToLoad", IsRequired = false, DefaultValue = "0")]
        public string GoBackDaysForStartDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("goBackDaysForStartDateToLoad") ? explicitValue["goBackDaysForStartDateToLoad"] : (string)this["goBackDaysForStartDateToLoad"] );
            }
            set
            {
                explicitValue["goBackDaysForStartDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("initialLoad", IsRequired = false, DefaultValue = "")]
        public string IsInitialLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("initialLoad") ? explicitValue["initialLoad"] : (string)this["initialLoad"] );
            }
            set
            {
                explicitValue["initialLoad"] = value;
            }
        }

        [ConfigurationProperty("nonInitialFullLoad", IsRequired = false, DefaultValue = "")]
        public string IsNonInitialFullLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("nonInitialFullLoad") ? explicitValue["nonInitialFullLoad"] : (string)this["nonInitialFullLoad"] );
            }
            set
            {
                explicitValue["nonInitialFullLoad"] = value;
            }
        }

        [ConfigurationProperty("startDateToLoad", IsRequired = false, DefaultValue = "")]
        public string StartDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("startDateToLoad") ? explicitValue["startDateToLoad"] : (string)this["startDateToLoad"] );
            }
            set
            {
                explicitValue["startDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("ssasStartDateToLoad", IsRequired = false, DefaultValue = "")]
        public string SSASStartDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("ssasStartDateToLoad") ? explicitValue["ssasStartDateToLoad"] : (string)this["ssasStartDateToLoad"] );
            }
            set
            {
                explicitValue["ssasStartDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("dimStartDateToLoad", IsRequired = false, DefaultValue = "")]
        public string DIMStartDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("dimStartDateToLoad") ? explicitValue["dimStartDateToLoad"] : (string)this["dimStartDateToLoad"] );
            }
            set
            {
                explicitValue["dimStartDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("endDateToLoad", IsRequired = false, DefaultValue = "")]
        public string EndDateToLoad
        {
            get
            {
                return ( explicitValue.ContainsKey("endDateToLoad") ? explicitValue["endDateToLoad"] : (string)this["endDateToLoad"] );
            }
            set
            {
                explicitValue["endDateToLoad"] = value;
            }
        }

        [ConfigurationProperty("cubeXmla", IsRequired = false, DefaultValue = "")]
        public string CubeXmla
        {
            get
            {
                return (string)this["cubeXmla"];
            }
        }

        [ConfigurationProperty("factAdjustmentsLoad", IsRequired = false, DefaultValue = "N")]
        public string FactAdjustmentsLoad
        {
            get
            {
                return (string)this["factAdjustmentsLoad"];
            }
        }

        [ConfigurationProperty("dimensionAuthorityDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string DimensionAuthorityDBConnectionString
        {
            get
            {
                return (string)this["dimensionAuthorityDBConnectionString"];
            }
        }

        [ConfigurationProperty("reportingDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string ReportingDBConnectionString
        {
            get
            {
                return (string)this["reportingDBConnectionString"];
            }
        }

        [ConfigurationProperty("destDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string DestDBConnectionString
        {
            get
            {
                return (string)this["destDBConnectionString"];
            }
        }

        [ConfigurationProperty("debug", IsRequired = false, DefaultValue = "N")]
        public string Debug
        {
            get
            {
                return ((string)this["debug"]).Substring(0,1);
            }
        }

        [ConfigurationProperty("dwDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string DWDBConnectionString
        {
            get
            {
                return (string)this["dwDBConnectionString"];
            }
        }

        [ConfigurationProperty("ssasSourceDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string SSASSourceDBConnectionString
        {
            get
            {
                return (string)this["ssasSourceDBConnectionString"];
            }
        }

        [ConfigurationProperty("ssasDestDBConnectionString", IsRequired = false, DefaultValue = null)]
        public string ssasDestDBConnectionString
        {
            get
            {
                return (string)this["ssasDestDBConnectionString"];
            }
        }

        [ConfigurationProperty("ssasServerName", IsRequired = false, DefaultValue = "")]
        public string SSASServerName
        {
            get
            {
                return (explicitValue.ContainsKey("ssasServerName") ? explicitValue["ssasServerName"] : (string)this["ssasServerName"]);
            }
            set
            {
                explicitValue["ssasServerName"] = value;
            }
        }

        [ConfigurationProperty("ssasProcessingServerName", IsRequired = false, DefaultValue = "")]
        public string SSASProcessingServerName
        {
            get
            {
                return (string)this["ssasProcessingServerName"];
            }
        }

        [ConfigurationProperty("sourceSsasServerName", IsRequired = false, DefaultValue = "")]
        public string SourceSSASServerName
        {
            get
            {
                return (string)this["sourceSsasServerName"];
            }
        }

        [ConfigurationProperty("targetSsasServerName", IsRequired = false, DefaultValue = "")]
        public string TargetSSASServerName
        {
            get
            {
                return (string)this["targetSsasServerName"];
            }
        }

        [ConfigurationProperty("ssasDBName", IsRequired = false, DefaultValue = "")]
        public string SSASDBName
        {
            get
            {
                return (explicitValue.ContainsKey("ssasDBName") ? explicitValue["ssasDBName"] : (string)this["ssasDBName"]);
            }
            set
            {
                explicitValue["ssasDBName"] = value;
            }
        }

        [ConfigurationProperty("maxDatesToLoad", IsRequired = false, DefaultValue = "")]
        public string MaxDatesToLoad
        {
            get
            {
                return (string)this["maxDatesToLoad"];
            }
        }

        [ConfigurationProperty("cubePermissionsConfiguration", IsRequired = false, DefaultValue = "")]
        public string CubePermissionsConfiguration
        {
            get
            {
                return (string)this["cubePermissionsConfiguration"];
            }
        }

        [ConfigurationProperty("sourceSsasDBName", IsRequired = false, DefaultValue = "")]
        public string SourceSSASDBName
        {
            get
            {
                return (string)this["sourceSsasDBName"];
            }
        }

        [ConfigurationProperty("targetSsasDBName", IsRequired = false, DefaultValue = "")]
        public string TargetSSASDBName
        {
            get
            {
                return (string)this["targetSsasDBName"];
            }
        }

        [ConfigurationProperty("sourceSsasCubeName", IsRequired = false, DefaultValue = "")]
        public string SourceSSASCubeName
        {
            get
            {
                return (string)this["sourceSsasCubeName"];
            }
        }

        [ConfigurationProperty("targetSsasCubeName", IsRequired = false, DefaultValue = "")]
        public string TargetSSASCubeName
        {
            get
            {
                return (string)this["targetSsasCubeName"];
            }
        }

        [ConfigurationProperty("ssasFullLoad", IsRequired = false, DefaultValue = "")]
        public string IsSSASFullLoad
        {
            get
            {
                return (explicitValue.ContainsKey("ssasFullLoad") ? explicitValue["ssasFullLoad"] : (string)this["ssasFullLoad"]);
            }
            set
            {
                explicitValue["ssasFullLoad"] = value;
            }
        }

        [ConfigurationProperty("ssasLoadCalculations", IsRequired = false, DefaultValue = "")]
        public string IsSSASLoadCalculations
        {
            get
            {
                return (string)this["ssasLoadCalculations"];
            }
        }

        [ConfigurationProperty("dbProviderName", IsRequired = false, DefaultValue = "No description.")]
        public string DBProviderName
        {
            get
            {
                return (string)this["dbProviderName"];
            }
        }

        [ConfigurationProperty("processedFilesDir", IsRequired = false, DefaultValue = "No description.")]
        public string ProcessedFilesDir
        {
            get
            {
                return (string)this["processedFilesDir"];
            }
        }

        [ConfigurationProperty("sparDBConnectionString", IsRequired = false, DefaultValue = "No description.")]
        public string SparDBConnectionString
        {
            get
            {
                return (string)this["sparDBConnectionString"];
            }
        }

        [ConfigurationProperty("sourceFilesDir", IsRequired = false, DefaultValue = null)]
        public string SourceFilesDir
        {
            get
            {
                return (string)this["sourceFilesDir"];
            }
        }

        [ConfigurationProperty("inputFile", IsRequired = false, DefaultValue = null)]
        public string InputFile
        {
            get
            {
                return (string)this["inputFile"];
            }
        }

        [ConfigurationProperty("inputConfigFile", IsRequired = false, DefaultValue = null)]
        public string InputConfigFile
        {
            get
            {
                return (string)this["inputConfigFile"];
            }
        }

        [ConfigurationProperty("outputDir", IsRequired = false, DefaultValue = null)]
        public string OutputDir
        {
            get
            {
                return (string)this["outputDir"];
            }
        }

        [ConfigurationProperty("dbImportFilesDir", IsRequired = false, DefaultValue = "No description.")]
        public string DbImportFilesDir
        {
            get
            {
                return (string)this["dbImportFilesDir"];
            }
        }

        [ConfigurationProperty("dbImportFilesShareOrExternalDir", IsRequired = false, DefaultValue = "No description.")]
        public string DbImportFilesShareOrExternalDir
        {
            get
            {
                return (string)this["dbImportFilesShareOrExternalDir"];
            }
        }

        public string GetConfigurationProperty(string propertyName)
        {
            return (explicitValue.ContainsKey(propertyName) ? explicitValue[propertyName] : (string)this[propertyName]);
        }

        public override string ToString()
        {
            string currentEnvironmentDesc = "Configuration name " + GetConfigurationProperty("name") + "\r\n\r\n";

            int maxNameLength = 0;
            var currentEnvironment = new Dictionary<string, string>();  
            foreach (ConfigurationProperty property in Properties)
            {
                if (property != null)
                {
                    if (property.Name == "name")
                        continue;

                    if (!String.IsNullOrEmpty(GetConfigurationProperty(property.Name))
                        && GetConfigurationProperty(property.Name) != (property.DefaultValue == null ? "" : property.DefaultValue.ToString()))
                    {
                        maxNameLength = Math.Max(maxNameLength, property.Name.Length);
                        currentEnvironment.Add(property.Name, GetConfigurationProperty(property.Name));
                    }
                }
            }

            foreach (KeyValuePair<string, string> kvp in currentEnvironment)
                currentEnvironmentDesc += String.Format("\t{0}{1} : {2}\r\n", kvp.Key, new string(' ', maxNameLength - kvp.Key.Length), kvp.Value);
 
            return currentEnvironmentDesc;                       
        }

    }
}