﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System.Configuration;

namespace DWJobs.Common.AppEnvironment
{
    internal class AppSettingsPerEnvironmentSection : System.Configuration.ConfigurationSection
    {
        private static ConfigurationProperty defaultEnvironment = new ConfigurationProperty("defaultEnvironment", typeof(string), "DEFAULT", ConfigurationPropertyOptions.IsRequired);

        public AppSettingsPerEnvironmentSection()  
        {
            base.Properties.Add(AppSettingsPerEnvironmentSection.defaultEnvironment);  
        }
  
        #region " Properties "  

        [StringValidator(MaxLength = 30, MinLength = 1, InvalidCharacters = @"!@#$%^&*()_-=+\|/?<>")]
        public string DefaultEnvironment  
        {  
            get 
            {
                return (string)this["defaultEnvironment"];  
            }  
            set 
            {
                this["defaultEnvironment"] = value;  
            }  
        }
        #endregion  

        public static AppSettingsPerEnvironmentSection Current
        {
            get
            {
                return (AppSettingsPerEnvironmentSection)ConfigurationManager.GetSection("appSettingsPerEnvironment");
            }
        }

        [ConfigurationProperty("appEnvironments", IsDefaultCollection = false),ConfigurationCollection(typeof(AppEnvironmentSettingsCollection), AddItemName = "addEnvironmentSettings", ClearItemsName = "clearEnvironmentSettings", RemoveItemName = "removeEnvironmentSettings")]
        public AppEnvironmentSettingsCollection AppEnvironmentSettings
        {
            get
            {
                return this["appEnvironments"] as AppEnvironmentSettingsCollection; 
            }
         }
    }
}
