﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.IO;

namespace DWJobs.Common
{
    public static class DatabaseHelper
    {
        public static string DumpSelectResultToCSVFile(string sqlSelect, string connectionString)
        {
            SqlDataAdapter sqlDataAdapter = new SqlDataAdapter(sqlSelect, connectionString);
            return DatabaseHelper.DumpSelectResultToCSVFile(sqlDataAdapter, null );
        }

        public static string DumpSelectResultToDelimitedFile(   SqlCommand  command                 , 
                                                                string      outputFileFullName      , 
                                                                string      dataFileFieldTerminator ,
                                                                string      dataFileRowTerminator   ,
                                                                bool        createRowHeader             )
        {
            if (String.IsNullOrEmpty(outputFileFullName))
                outputFileFullName = Path.GetTempFileName();

            using (FileStream fileStream = File.Create(outputFileFullName))
            {
                using (StreamWriter streamWriter = new StreamWriter(fileStream))
                {
                    using ( SqlDataReader reader = command.ExecuteReader() )
                    {
                        bool readingFirstRow = true;
                        bool dataHasBitColumns = false;
                        bool[] indexesOfBitColumns = new bool[1000];

                        while( reader != null && reader.Read() )
                        {
                            if ( readingFirstRow )
                                for (int valIndex = 0; valIndex < reader.VisibleFieldCount; valIndex++)
                                {
                                    if (reader.GetDataTypeName(valIndex) == "bit")
                                    {
                                        dataHasBitColumns = true;
                                        indexesOfBitColumns[valIndex] = true;
                                    }
                                 }

                            if (readingFirstRow && createRowHeader)
                            {
                                StringBuilder rowHeader = new StringBuilder();
                                for (int valIndex = 0; valIndex < reader.VisibleFieldCount; valIndex++)
                                {
                                    if (valIndex != 0)
                                        rowHeader.Append(dataFileFieldTerminator);
                                    rowHeader.Append(reader.GetName(valIndex).ToString());
                                }
                                streamWriter.Write(rowHeader.ToString() + dataFileRowTerminator);
                            }

                            StringBuilder rowData = new StringBuilder();
                            for (int valIndex = 0; valIndex < reader.VisibleFieldCount; valIndex++)
                            {
                                if (valIndex != 0)
                                    rowData.Append(dataFileFieldTerminator);
                                if ( dataHasBitColumns && indexesOfBitColumns[valIndex] && !reader.IsDBNull(valIndex ) )
                                    rowData.Append( reader.GetValue(valIndex ).ToString() == "False" ? "0" : "1" );
                                else
                                    rowData.Append( reader.GetValue(valIndex ).ToString() );
                            }
                            streamWriter.Write(rowData.ToString() + dataFileRowTerminator);

                            readingFirstRow = false;
                        }
                    }
                }
            }

            return outputFileFullName;
        }

        public static string DumpSelectResultToCSVFile(SqlCommand command, string filePath)
        {
            SqlDataAdapter sqlDataAdapter = new SqlDataAdapter(command);
            return DatabaseHelper.DumpSelectResultToCSVFile(sqlDataAdapter, filePath);
        }

        private static string DumpSelectResultToCSVFile(SqlDataAdapter adapter, string filePath)
        {
            if (String.IsNullOrEmpty( filePath ) )
                filePath = Path.GetTempFileName();

            DataSet dataset = new DataSet();

            adapter.Fill(dataset);

            if (dataset.Tables.Count > 0 )
            {
                using (FileStream fileStream = File.Create(filePath))
                {
                    using (StreamWriter streamWriter = new StreamWriter(fileStream))
                    {
                        StringBuilder rowHeader = new StringBuilder();
                        for (int colIndex = 0; colIndex < dataset.Tables[0].Columns.Count; colIndex++)
                        {
                            if (colIndex != 0)
                                rowHeader.Append(",");
                            rowHeader.Append(dataset.Tables[0].Columns[colIndex].Caption);
                        }
                        streamWriter.WriteLine(rowHeader.ToString());

                        foreach (DataRow row in dataset.Tables[0].Rows)
                        {
                            StringBuilder rowData = new StringBuilder();
                            for (int valIndex = 0; valIndex < row.ItemArray.GetLength(0); valIndex++)
                            {
                                if (valIndex != 0)
                                    rowData.Append(",");
                                if (row[valIndex].GetType() == typeof(bool) )
                                    rowData.Append( row[valIndex].ToString().ToLower().Replace("true", "1").Replace( "false", "0") );
                                else if (row[valIndex].ToString().Contains(",") || row[valIndex].ToString().Contains("\"") || row[valIndex].ToString().Contains("\n") )
                                    rowData.Append("\"" + row[valIndex].ToString().Replace("\"", "\"\"") + "\"");
                                else
                                    rowData.Append(row[valIndex].ToString());
                            }
                            streamWriter.WriteLine(rowData.ToString());
                        }
                    }
                }
            }
            else
            {
                using (StreamWriter streamWriter = File.CreateText(filePath))
                {
                    streamWriter.Close();
                }

            }

            return filePath;
        }

        public static List<string> DumpSelectResultToList(string sqlSelect, string connectionString)
        {
            SqlDataAdapter sqlDataAdapter = new SqlDataAdapter(sqlSelect, connectionString);

            List<string> list = new List<string>();

            DataSet dataset = new DataSet();

            sqlDataAdapter.Fill(dataset);

            StringBuilder rowHeader = new StringBuilder();
            for (int colIndex = 0; colIndex < dataset.Tables[0].Columns.Count; colIndex++)
            {
                if (colIndex != 0)
                    rowHeader.Append(",");
                rowHeader.Append(dataset.Tables[0].Columns[colIndex].Caption);
            }
            list.Add(rowHeader.ToString());

            foreach (DataRow row in dataset.Tables[0].Rows)
            {
                StringBuilder rowData = new StringBuilder();
                for (int valIndex = 0; valIndex < row.ItemArray.GetLength(0); valIndex++)
                {
                    if (valIndex != 0)
                        rowData.Append(",");
                    rowData.Append(row[valIndex].ToString());
                }
                list.Add(rowData.ToString());
            }

            return list;
        }
    }
}
