﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System.Collections.Generic;
using System.Diagnostics;

namespace DWJobs.Common
{
    public class ExcelProcessHelper
    {
        /// <summary>
        /// kill the process of the input procId
        /// </summary>
        /// <param name="procId">input running process ID</param>
        public static void KillProcess(int procId)
        {
            if (procId > 0)
                Process.GetProcessById(procId).Kill();
        }

        /// <summary>
        /// Get the process IDs of the current running Excel processes
        /// </summary>
        /// <returns>A list of running Excel process IDs</returns>
        public static List<int> GetCurrentExcelProceses()
        {
            List<int> proceesIdsCurrentExcelProcesses = new List<int>();

            // Get the current processes
            Process[] runningProcesses = Process.GetProcesses();

            foreach (Process current in runningProcesses)
            {
                // Check for a match.
                if (current.ProcessName.ToUpper() == "EXCEL")
                    proceesIdsCurrentExcelProcesses.Add(current.Id);
                current.Dispose();
            }
            return proceesIdsCurrentExcelProcesses;
        }

        /// <summary>
        /// Get the Process Id of a running excel which is NOT in the input list
        /// </summary>
        /// <param name="proceesIdsCurrentExcelProcesses">input list of running Excel process IDs</param>
        /// <returns>the new process id if >0, else if -1 indicating there is no new process Id</returns>
        public static int GetIdOfNewExcelProcess(List<int> proceesIdsCurrentExcelProcesses)
        {
            int newExcelProceesId = -1;
            Process[] runningProcesses = Process.GetProcesses();

            foreach (Process current in runningProcesses)
            {
                // Check for a match.
                if (current.ProcessName.ToUpper() == "EXCEL" && proceesIdsCurrentExcelProcesses.BinarySearch(current.Id) < 0)
                    newExcelProceesId = current.Id;
                current.Dispose();
                if (newExcelProceesId > -1)
                    break;
            }

            return newExcelProceesId;
        }

    }
}
