﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Net.Mail;
using System.IO;
using log4net;

namespace DWJobs.Common
{
    public class Mailer
    {
        public static void SendMailMessage(string from, string to, string subject, string body, Dictionary<string, string> attachmentFilePathAndUserName, ILog log)
        {

			// Set mailServerName to be the name of the mail server
			// you wish to use to deliver this message
            string mailServerName = "mailhost.....";  

            try
            {
                // MailMessage is used to represent the e-mail being sent
                using (MailMessage message = new MailMessage(from, to, subject, body))
                {
                    if ( attachmentFilePathAndUserName != null )
                        foreach (KeyValuePair<string, string> kvp in attachmentFilePathAndUserName)
                            Mailer.AddAttachment(message, kvp.Value, kvp.Key);

                    // SmtpClient is used to send the e-mail
					SmtpClient mailClient = new SmtpClient(mailServerName);

					// UseDefaultCredentials tells the mail client to use the 
                    // Windows credentials of the account (i.e. user account) 
                    // being used to run the application
                    mailClient.UseDefaultCredentials = true;

                    log.Info(String.Format( "Sending e-mail message  From: {0}  To: {1}  Subject: {2}", from, to, subject ) );

                    // Send delivers the message to the mail server
                    mailClient.Send(message);
                }
                log.Info("e-mail message sent.");
            }
            catch (FormatException ex)
            {
                log.Error("Cannot send e-mail message", ex);
            }
            catch (SmtpException ex)
            {
                log.Error("Cannot send e-mail message", ex);
            }
		}

        private static void AddAttachment( MailMessage message, string fileFullName, string fileNameInMail )
        {
            FileStream fileStream = new FileStream(fileFullName, FileMode.Open, FileAccess.Read);

            Attachment data = new Attachment(fileStream, fileNameInMail);

            message.Attachments.Add(data);
        }

    }
}
