﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using DWJobs.Common.AppEnvironment;

namespace DWJobs.DataSourceLoader
{
    public static class FeedProcessorFactory
    {

        public static IProcessFeed Create(TableLoadAction tableLoadAction, StagingDatabase db)
        {
            if (tableLoadAction.DataSourceType == TableLoadAction.LoaderDataSourceType.FileDataSource)
                return new FileLoader(tableLoadAction, db, AppConfiguration.Log);
            else if (tableLoadAction.DataSourceType == TableLoadAction.LoaderDataSourceType.SqlDataSource)
                return new SqlServiceLoader(tableLoadAction, db, AppConfiguration.IsNonInitialFullLoad, AppConfiguration.Log);
            else if (tableLoadAction.DataSourceType == TableLoadAction.LoaderDataSourceType.SourceLoaderDataSource)
                return new SourceLoaderTaskExecutor(tableLoadAction, db, AppConfiguration.Log);

            throw new InvalidOperationException("Unsupported data source: " + tableLoadAction.DataSourceType.ToString());
        }
    }
}
