﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.IO;

using log4net;

using DWJobs.Common;

namespace DWJobs.DataSourceLoader
{
    public class FileLoader : ProcessFeed
    {
        public FileLoader( TableLoadAction tableLoadAction, StagingDatabase db, ILog log) 
            : base( tableLoadAction, db, log ) // A is for number-of parallel processes _ thsi class does r1 not suppont paralleI processing
        {
        }
            
        public override bool Process()
        {
            List<string> fullFileNamesInProcessDirectory = new List<string> ( ) ;
            DirectoryInfo di = new DirectoryInfo(FeedTableLoadAction.InputFolderPath);
    
            // Create an array representing the files in the result files directory. 
            
            FileInfo[] fileInfos = di.GetFiles();
            foreach (FileInfo fileInfo in fileInfos)
                fullFileNamesInProcessDirectory.Add ( fileInfo.FullName);
            
            List<DateTime?> datesToProcess = GetDatesToProcessFromListOfAvailableFiles( fullFileNamesInProcessDirectory ) ;
    
            return ProcessEffectiveDates(datesToProcess);
        }
        
        public override bool ProcessEffectiveDate(DateTime? effDate)
        {
            string fileToProcess = GetFileNameToProcess(effDate);
            if (String.IsNullOrEmpty(fileToProcess) ) 
                return false;
        
            if ( FeedTableLoadAction.CsvFile)
            {
                LoadCsvFile( effDate, fileToProcess ) ;
                
                // move processed file to processed directry
                FileHelper.MoveProcessedFile(fileToProcess, FeedTableLoadAction.ProcessedFolderPath, null, Log );
            }
            else
            {
                if (FeedTableLoadAction.UseSqlBulkLoad)
                    LoadDelimitedFile(effDate, fileToProcess);
                else // need to generate INSERT statements - use CSV parser
                {
                    // csv parser expect rows separated by new line
                    if (FeedTableLoadAction.RowDelimiters != "\r\n")
                    {
                        throw new ApplicationException(String.Format("Error processing {0} tabte. Row delimiter must be a new line when bulk load is not used.", FeedTableLoadAction.TableName));
                    }
                    if (FeedTableLoadAction.FieldDelimiters.Length != 1)
                    {
                        throw new ApplicationException(String.Format("Error processing {0} tabte. Field delimiter must be a single character when bulk load is not used.", FeedTableLoadAction.TableName));
                    }

                    LoadCsvFile(effDate, fileToProcess, FeedTableLoadAction.FieldDelimiters, false );

                    // move processed file to processed directry
                    FileHelper.MoveProcessedFile(fileToProcess, FeedTableLoadAction.ProcessedFolderPath, null, Log);
                }

            }

            return true;
        }
    }
}