﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using DWJobs.Common;
using System.Globalization;

using log4net;

namespace DWJobs.DataSourceLoader
{
    public class StagingDatabase
    {
        private readonly string connectionString;
        private string providerName;
        private string debug;
        private readonly ILog log;
        private string printText;
        
        private const int RESULT_SUCCESS = 0;
        
        public StagingDatabase(string connectionString, string providerName, ILog log, string debug )
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.log = log;
            this.debug = debug;
        }

        public string ProcessFile(  string          dataFilePath ,
                                    string          insertStatements,
                                    string          dataFileColumnList ,
                                    string          dataFileFieldTerminator ,
                                    string          dataFileRowTerminator ,
                                    DateTime?       effectiveDate ,
                                    string          effectiveDateColumnName ,
                                    string          destTableSchema ,
                                    string          destTableName , 
                                    bool            isTruncateDestTable ,
                                    bool            isUseBulkInsert ,
                                    string          postEffDateLoadProcessingSQL ,
                                    string          effectiveDatePlaceHolderString ,
                                    string          processedBy )
        {
            int returnValue = -1;

            this.log.InfoFormat("Processing file {0}", dataFilePath );
            string queryString = "[gen].[Load Data Source]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler( InfoMessageHandler) ;

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;
                
                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data. ParameterDirection.ReturnValue;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@DataFilePath", SqlDbType.Char);
                if (String.IsNullOrEmpty(dataFilePath))
                    param.Value = DBNull.Value;
                else
                    param.Value = dataFilePath;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@InsertStatements", SqlDbType.NVarChar);
                if (String.IsNullOrEmpty(insertStatements))
                    param.Value = DBNull.Value;
                else
                    param.Value = insertStatements;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter( "@DataFileColumnList", SqlDbType.Char) ;
                if (String.IsNullOrEmpty(dataFileColumnList) )
                    param.Value = DBNull.Value;
                else
                    param.Value = dataFileColumnList;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters. Add ( param) ;

                param = new SqlParameter( "@DataFileFieldTerminator", SqlDbType.Char) ;
                if (String.IsNullOrEmpty ( dataFileFieldTerminator) )
                    param.Value = DBNull.Value;
                else
                    param.Value = dataFileFieldTerminator;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;
                
                param = new SqlParameter("@DataFileRowTerminator", SqlDbType.Char);
                if (String. IsNullOrEmpty(dataFileRowTerminator) )
                    param.Value = DBNull.Value;
                else
                    param.Value = dataFileRowTerminator;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param) ;
                
                param = new SqlParameter("@EffectiveDate", SqlDbType.Date);
                if (effectiveDate == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = effectiveDate.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);

                param = new SqlParameter("@EffectiveDateColumnName", SqlDbType.Char) ;
                if (String.IsNullOrEmpty(effectiveDateColumnName) )
                    param.Value = DBNull.Value;
                else
                    param.Value = effectiveDateColumnName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@DestTableSchema", SqlDbType.Char);
                param.Value = destTableSchema;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@DestTableName", SqlDbType.Char);
                param.Value = destTableName;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param) ;
                
                param = new SqlParameter("@IsTruncateDestTable", SqlDbType.Bit) ;
                param.Value = isTruncateDestTable ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);
                
                param = new SqlParameter("@IsUseBulkInsert", SqlDbType.Char);
                param.Value = isUseBulkInsert ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add (param) ;
                
                param = new SqlParameter("@PostEffDateLoadProcessingSQL", SqlDbType.Char);
                if (String. IsNullOrEmpty( postEffDateLoadProcessingSQL) )
                    param.Value = DBNull.Value;
                else
                    param.Value = postEffDateLoadProcessingSQL;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add (param);

                param = new SqlParameter( "@EffectiveDatePlaceHolderString", SqlDbType. Char); 
                if (String. IsNullOrEmpty(effectiveDatePlaceHolderString))
                    param.Value = DBNull.Value;
                else
                    param.Value = effectiveDatePlaceHolderString;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters.Add (param) ;
                
                param = new SqlParameter("@ProcessedBy", SqlDbType.Char);
                if (processedBy == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = processedBy;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                try
                {
                    connection.Open();
                    this.printText = "";

                    command. ExecuteScalar( ) ;
                    
                    returnValue = Int32.Parse( command.Parameters[ "@Retval" ].Value.ToString( ) );
                    
                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String. Format ( "Error processing file" ) ) ;
                    this.log.InfoFormat("Finished processing file {0}", dataFilePath ); 
                    this.log.InfoFormat ( "" ) ;
                    return "";
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                    
                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string DoPostLoadProcessing( DateTime?       firstEffectiveDateProcessed             ,
                                            DateTime?       lastEffectiveDateProcessed              ,
                                            string          postLoadProcessingSQL                   ,
                                            string          firstEffectiveDatePlaceHolderString     ,
                                            string          lastEffectiveDatePlaceHolderString      ,
                                            string          processedBy                                 )
        {
            int returnValue = -1;

            this.log.InfoFormat("Running post Load processing" );
            string queryString = "[gen].[Do Post Data Source Load Processing]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {

                connection.InfoMessage += new SqlInfoMessageEventHandler( InfoMessageHandler) ;

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;
                
                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data. ParameterDirection.ReturnValue;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@FirstEffectiveDateProcessed", SqlDbType.Date);
                if (firstEffectiveDateProcessed == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = firstEffectiveDateProcessed.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);
                
                param = new SqlParameter("@LastEffectiveDateProcessed", SqlDbType.Date);
                if (lastEffectiveDateProcessed == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = lastEffectiveDateProcessed.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);
                
                param = new SqlParameter("@PostLoadProcessingSQL", SqlDbType.Char);
                if (postLoadProcessingSQL == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = postLoadProcessingSQL;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@FirstEffectiveDatePlaceHolderString", SqlDbType.Char);
                if (firstEffectiveDatePlaceHolderString == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = firstEffectiveDatePlaceHolderString;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@LastEffectiveDatePlaceHolderString", SqlDbType.Char);
                if (lastEffectiveDatePlaceHolderString == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = lastEffectiveDatePlaceHolderString;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;

                param = new SqlParameter("@ProcessedBy", SqlDbType.Char);
                if (processedBy == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = processedBy;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                try
                {
                    connection.Open();
                    this.printText = "";

                    command. ExecuteScalar( ) ;
                    
                    returnValue = Int32.Parse( command.Parameters[ "@Retval" ].Value.ToString( ) );
                    
                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String. Format ( "Error running post processing file" ) ) ;
                    this.log.InfoFormat("Finished post load processing" ); 
                    this.log.InfoFormat ( "" ) ;
                    return "";
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                    
                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string TruncateTable(    string destTableSchema,
                                        string destTableName    )
        {
            int returnValue = -1;

            this.log.InfoFormat("Truncating table {0}.{1}", destTableSchema, destTableName);
            string queryString = String.Format( "TRUNCATE TABLE [{0}].[{1}]", destTableSchema, destTableName);

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {

                connection.InfoMessage += new SqlInfoMessageEventHandler( InfoMessageHandler) ;

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.Text;
                command.CommandTimeout = 0;
                
                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data. ParameterDirection.ReturnValue;
                command.Parameters.Add( param) ;
                
                try
                {
                    connection.Open();
                    this.printText = "";

                    command. ExecuteScalar( ) ;
                    
                    returnValue = Int32.Parse( command.Parameters[ "@Retval" ].Value.ToString( ) );
                    
                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String. Format ("Error truncating table") ) ;
                    this.log.InfoFormat("Finished truncating table {0}.{1}", destTableSchema, destTableName);
                    this.log.InfoFormat ( "" ) ;
                    return "";
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                    
                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GetNextDateToLoad(    string effectiveDateColumnName  ,     
                                            string destTableSchema          , 
                                            string destTableName            ,
                                            out DateTime? nextEffectiveDate )
        {
            int returnValue = -1;

            this.log.InfoFormat("Determining next date to Load for {0}.{1}", destTableSchema, destTableName );
            string queryString = "[gen].[Get Next Date To Load Data Source]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {

                connection.InfoMessage += new SqlInfoMessageEventHandler( InfoMessageHandler) ;

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;
                
                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data. ParameterDirection.ReturnValue;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@NextEffectiveDate", SqlDbType.Date);
                param.Direction = System.Data.ParameterDirection.Output;
                command. Parameters. Add ( param);

                param = new SqlParameter("@EffectiveDateColumnName", SqlDbType.Char);
                if (effectiveDateColumnName == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = effectiveDateColumnName;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);
                
                param = new SqlParameter("@DestTableSchema", SqlDbType.Char);
                if (destTableSchema == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = destTableSchema;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);

                param = new SqlParameter("@DestTableName", SqlDbType.Char);
                if (destTableName == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = destTableName;
                param.Direction = System.Data.ParameterDirection.Input;
                command. Parameters. Add ( param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                try
                {
                    connection.Open();
                    this.printText = "";

                    command. ExecuteScalar( ) ;

                    nextEffectiveDate = ( command.Parameters[ "@NextEffectiveDate" ].Value == DBNull.Value ? null : new DateTime? ( (DateTime)command.Parameters["@NextEffectiveDate"].Value ));

                    returnValue = Int32.Parse( command.Parameters[ "@Retval" ].Value.ToString( ) );
                    
                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String. Format ( "Error determining next effective date to load" ) ) ;
                    this.log.InfoFormat("Finished determining next effective date to load. It is - {0}", ( nextEffectiveDate == null ? "null" : nextEffectiveDate.Value.ToShortDateString() ) ); 
                    this.log.InfoFormat ( "" ) ;
                    return "";
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                    
                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string ExecuteStoredProcedure(   string                  connectionString         ,
                                                string                  sql                      ,
                                                bool                    isStoredProcedure        ,
                                                List<ProcParameter>     procedureParameters         )
        {
            return ExecuteStoredProcedureSaveResult(null,
                                                    false,
                                                    connectionString,
                                                    sql,
                                                    isStoredProcedure,
                                                    procedureParameters,
                                                    null,
                                                    null);

        }

        public string ExecuteStoredProcedureSaveResultToCsv(string                  resultFileNamePath,
                                                            string                  connectionString         ,
                                                            string                  sql                      ,
                                                            bool                    isStoredProcedure        ,
                                                            List<ProcParameter>     procedureParameters         )
        {
            return ExecuteStoredProcedureSaveResult(resultFileNamePath,
                                                    true,
                                                    connectionString,
                                                    sql,
                                                    isStoredProcedure,
                                                    procedureParameters,
                                                    null,
                                                    null);

        }

        public string ExecuteStoredProcedureSaveResultToDelimitedFile(  string                  resultFileNamePath,
                                                                        string                  connectionString         ,
                                                                        string                  sql                      ,
                                                                        bool                    isStoredProcedure        ,
                                                                        List<ProcParameter>     procedureParameters      ,
                                                                        string                  fieldDelim               ,
                                                                        string                  rowDelim        )
        {
            return ExecuteStoredProcedureSaveResult(resultFileNamePath,
                                                    true,
                                                    connectionString,
                                                    sql,
                                                    isStoredProcedure,
                                                    procedureParameters,
                                                    fieldDelim ,
                                                    rowDelim);

        }

        public string ExecuteStoredProcedureSaveResult( string                  resultFileNamePath       ,
                                                        bool                    isSaveToCsv              ,
                                                        string                  connectionString         ,
                                                        string                  sql                      ,
                                                        bool                    isStoredProcedure        ,
                                                        List<ProcParameter>     procedureParameters      ,
                                                        string                  fieldDelim               ,
                                                        string                  rowDelim            )
        {

            int returnValue = -1;
            this.log.InfoFormat("Processing sql {0}", sql);

            string queryString = sql;

            using (SqlConnection connection = new SqlConnection(connectionString))
            {

                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = (isStoredProcedure ? CommandType.StoredProcedure : CommandType.Text);
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                if (isStoredProcedure)
                    foreach (var procParam in procedureParameters)
                    {
                        SqlDbType sqlDbType;
                        switch (procParam.ParamType)
                        {
                            case "DATE":
                                sqlDbType = SqlDbType.Date;
                                break;
                            case "DATETIME":
                                sqlDbType = SqlDbType.DateTime;
                                break;
                            case "VARCHAR":
                                sqlDbType = SqlDbType.VarChar;
                                break;
                            case "NVARCHAR":
                                sqlDbType = SqlDbType.NVarChar;
                                break;
                            case "INT":
                                sqlDbType = SqlDbType.Int;
                                break;
                            case "SMALLINT":
                                sqlDbType = SqlDbType.SmallInt;
                                break;
                            case "BIGINT":
                                sqlDbType = SqlDbType.BigInt;
                                break;
                            default:
                                throw new ArgumentOutOfRangeException("Procedure parameter type " + procParam.ParamType + " is not currently supported. Modify TableActionDefinitlons.xsd and code here to add new types");
                        }
                        param = new SqlParameter(procParam.ParamName, sqlDbType);

                        param.Value = null;

                        if (procParam.ParamValue == "NULL" || String.IsNullOrEmpty(procParam.ParamValue))
                            param.Value = DBNull.Value;
                        else if (procParam.ParamType == "VARCHAR" || procParam.ParamType == "NVARCHAR")
                            param.Value = procParam.ParamValue;
                        else if (procParam.ParamType == "DATE" || procParam.ParamType == "DATETIME")
                        {
                            string[] dateFormats = { "yyyyMMdd", "yyyy-MM-dd", "M/d/yyyy",
                                                     "yyyyMMdd hh:mm:ss tt", "yyyy-MM-dd hh:mm:ss tt", "M/d/yyyy hh:mm:ss tt",
                                                     "yyyyMMdd HH:mm:ss", "yyyy-MM-dd HH:mm:ss", "M/d/yyyy HH:mm:ss" };
                            DateTime date;

                            if (DateTime.TryParseExact(procParam.ParamValue, dateFormats, CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                                param.Value = date;
                            else
                                throw new ApplicationException(String.Format("Error loading from {0} stored procedure. Could not convert parameter {1} date value {2} to a {3}", queryString, procParam.ParamName, procParam.ParamValue, procParam.ParamType));
                        }
                        else if (procParam.ParamType == "INT" || procParam.ParamType == "SMALLINT" || procParam.ParamType == "BIGINT")
                            param.Value = Int64.Parse(procParam.ParamValue);
                        param.Direction = ParameterDirection.Input;

                        command.Parameters.Add(param);
                    }

                try
                {
                    connection.Open();
                    this.printText = "";

                    if ( !String.IsNullOrEmpty(resultFileNamePath))
                    {
                        if (isSaveToCsv)
                            DatabaseHelper.DumpSelectResultToCSVFile(command, resultFileNamePath);
                        else
                            DatabaseHelper.DumpSelectResultToDelimitedFile(command, resultFileNamePath, fieldDelim, rowDelim, false);
                    }
                    else
                    {
                        command.ExecuteNonQuery();
                    }

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error determining next effective date to load"));
                    if (!String.IsNullOrEmpty(resultFileNamePath))
                        this.log.InfoFormat("Finished creating file {0}", resultFileNamePath);
                    this.log.InfoFormat("");
                    return "";
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText);

                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624)
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }


        private void InfoMessageHandler(object mySender, SqlInfoMessageEventArgs myEvent)
        {
            for (int i = 0; i < myEvent.Errors.Count; i++)
            {
                if ( this.log != null )
                    this.log.Info( myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") );
                else
                    this.printText += myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") + "\r\n";
            }

        }

        public string UpdateStatisticsInStaging(bool recompile)
        {
            int returnValue = -1;
            this.log.Info("Running update statistics");

            string queryString = "[stg].[Update Statistics In Staging DB]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Recompile", SqlDbType.Bit);
                param.Value = recompile ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    //                    this.printText = "";
                    command.ExecuteScalar();
                    //                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error ruuning Update Statistics In Staging DB"));

                    this.log.InfoFormat("Finished running Update Statistics In Staging DB");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }
    }
}