﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System.Collections.Generic;

namespace DWJobs.DataSourceLoader
{

    public class TableLoadAction
    {

        public enum LoaderDataSourceType
        {
            FileDataSource = 0,
            SqlDataSource = 1,
            SourceLoaderDataSource = 2
        };

        /// <summary>
        /// whether the action is performed or ignored
        /// </summary>
        public bool Active
        {
            get;
            set;
        }

        public string DatabaseName
        {
            get;
            set;
        }

        public string SchemaName
        {
            get;
            set;
        }

        public string TableName
        {
            get;
            set;
        }

        public string InputFolderPath
        {
            get;
            set;
        }

        /// <summary>
        /// When set, forces the loader to use BULKLOAD else use INSERT SQL statements
        /// </summary>
        public bool UseSqlBulkLoad
        {
            get;
            set;
        }

        /// <summary>
        /// When set, forces the loader to use direct SQL statements
        /// </summary>
        public bool UseDirectSql
        {
            get;
            set;
        }

        /// <summary>
        /// when set, a column [row number] will be added to the file to be loaded
        /// <summary>
        public bool PrependRowNumberColumn
        {
            get;
            set;
        }

        public string StartDate
        {
            get;
            set;
        }

        public string EndDate
        {
            get;
            set;
        }


        /// <summary>
        /// file name template, excludes path component
        /// </summary>
        public string FileNameTemplate
        {
            get;
            set;
        }

        public string DateKeyColumn
        {
            get;
            set;
        }

        public string DropFolderPath
        {
            get;
            set;
        }

        public string ProcessedFolderPath
        {
            get;
            set;
        }

        public string SsasConnectionString
        {
            get;
            set;
        }

        public int NumberOfParallelProcesses
        {
            get;
            set;
        }

        public string FieldDelimiters
        {
            get;
            set;
        }

        public string RowDelimiters
        {
            get;
            set;
        }

        public bool CsvFile
        {
            get;
            set;
        }

        public bool HasHeaderRow
        {
            get;
            set;
        }

        public string HeaderRowReplacement
        {
            get;
            set;
        }

        public bool SourceHasAllTargetTableColumnsInTheSameOrder
        {
            get;
            set;
        }

        public LoaderDataSourceType DataSourceType
        {
            get;
            set;
        }

        /// <summary>
        /// when set, blow away existing data for the current date key
        /// </summary>
        public bool ReplaceExisting
        {
            get;
            set;
        }

        public bool TruncateExisting
        {
            get;
            set;
        }

        public int HeaderLinesToSkip
        {
            get;
            set;
        }

        public List<FieldTransformation> FieldTransformations
        {
            get;
            set;
        }

        /// <summary>
        /// a list of source columns to be ignored
        /// </summary>
        public List<string> IgnoredInputColumns
        {
            get;
            set;
        }

        /// <summary>
        /// a list of source columns to be included. 
        /// Will be used when number of columns is long and number of needed columns is small
        /// </summary>
        public List<string> IncludedInputColumns
        {
            get;
            set;
        }

        /// <sunmary>
        /// a List of sQL to execute after single effective date is loaded 
        /// </sunmary>
        public List<string> PostEffDateLoadAction
        {
            get;
            set;
        }

        /// <sunmary>
        /// a List of SQL to execute before table load starts 
        /// </sunmary>
        public List<string> PreLoadAction
        {
            get;
            set;
        }

        /// <sunmary>
        /// a List of SQL to execute after all dates are loaded 
        /// </sunmary>
        public List<string> PostLoadAction
        {
            get;
            set;
        }


        /// <summary>
        /// translates column headers in the input file to column names in dest table
        /// </summary>
        public Dictionary<string, string> FieldNameMap
        {
            get;
            set;
        }

        /// <summary>
        /// true if this action is for a flat file data source
        /// </summary>
        public bool HasFlatFileDataSource
        {
            get
            {
                return DataSourceType == LoaderDataSourceType.FileDataSource;
            }
        }

        public List<string> PostLoadActions
        {
            get;
            set;
        }

        public string ConnectionString
        {
            get;
            set;
        }

        public string Sql
        {
            get;
            set;
        }

        /// <summary>
        /// When set, the SQL for SQLLoader is Stored procedure as opposed to select
        /// </summary>
        public bool IsStoredProcedure
        {
            get;
            set;
        }


        public List<ProcParameter> ProcedureParameters
        {
            get;
            set;
        }
    
        public string EffectiveDateField
        {
            get;
            set;
        }
    
    }

}
