﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace DWJobs.TabularSetsComparer.InputConfigXml
{
    public static class CompareScenarioImporter
    {
        public static IEnumerable<CompareScenario> LoadConfigurationDocument(string strFilePath)
        {
            var doc = XDocument.Load(strFilePath);
            var ns = doc.Root.Name.Namespace;
            var root = doc.Root.Descendants(ns + "CompareScenarios");
            var elements = root.Elements(ns + "CompareScenario");
            var results = elements.Select(xel => ElementToTableLoadAction(ns, xel));
            return results.Where(tla => tla.Active);
        }

        static private CompareScenario ElementToTableLoadAction(XNamespace ns, XElement xel)
        {
            var result = new CompareScenario()
            {
                Active = Boolean.Parse(xel.Element(ns + "Active").Attribute("value").Value),
                ScenarioName = xel.Element(ns + "ScenarioName").Attribute("value").Value,
                ExcludeFromCompareColumns = null,
                ScenarioType = CompareScenario.CompareScenarioType.SqlResultSetsCompare,
                FloatCompareThreshold = 0
            };

            if (xel.Element(ns + "StartDate") != null)
                result.StartDate = xel.Element(ns + "StartDate").Attribute("value").Value;
            if (xel.Element(ns + "EndDate") != null)
                result.EndDate = xel.Element(ns + "EndDate").Attribute("value").Value;

            if (xel.Element(ns + "AliasLeft") != null)
                result.AliasLeft = xel.Element(ns + "AliasLeft").Attribute("value").Value;
            if (xel.Element(ns + "AliasRight") != null)
                result.AliasRight = xel.Element(ns + "AliasRight").Attribute("value").Value;

            if (xel.Element(ns + "CompareResultsFolderPath") != null)
                result.CompareResultsFolderPath = xel.Element(ns + "CompareResultsFolderPath").Attribute("value").Value;
            if (xel.Element(ns + "FloatCompareThreshold") != null)
                result.FloatCompareThreshold = Double.Parse(xel.Element(ns + "FloatCompareThreshold").Attribute("value").Value);

            var dsElement = xel.Element(ns + "SpecificParameters");
            if (dsElement.Element(ns + "SqlResultSetsCompare") != null)
            {
                #region
                result.ScenarioType = CompareScenario.CompareScenarioType.SqlResultSetsCompare;

                var websElement = dsElement.Element(ns + "SqlResultSetsCompare");

                if (websElement.Element(ns + "QueryLeft") != null)
                    result.QueryLeft = websElement.Element(ns + "QueryLeft").Attribute("value").Value;
                if (websElement.Element(ns + "QueryRight") != null)
                    result.QueryRight = websElement.Element(ns + "QueryRight").Attribute("value").Value;

                #endregion
            }
            else if (dsElement.Element(ns + "MdxResultSetsCompare") != null)
            {
                #region
                result.ScenarioType = CompareScenario.CompareScenarioType.MdxResultsSetsCompare;

                var websElement = dsElement.Element(ns + "MdxResultSetsCompare");

                if (websElement.Element(ns + "QueryLeft") != null)
                    result.QueryLeft = websElement.Element(ns + "QueryLeft").Attribute("value").Value;
                if (websElement.Element(ns + "QueryRight") != null)
                    result.QueryRight = websElement.Element(ns + "QueryRight").Attribute("value").Value;
                else
                    result.QueryRight = result.QueryLeft;

                if (websElement.Element(ns + "SSASConnectionStringLeft") != null)
                    result.SSASConnectionStringLeft = websElement.Element(ns + "SSASConnectionStringLeft").Attribute("value").Value;
                if (websElement.Element(ns + "SSASConnectionStringRight") != null)
                    result.SSASConnectionStringRight = websElement.Element(ns + "SSASConnectionStringRight").Attribute("value").Value;

                #endregion
            }

            if (xel.Element(ns + "ResultSetColumnsAndTypes") != null)
                result.ResultSetColumnsAndTypes = xel.Element(ns + "ResultSetColumnsAndTypes").Attribute("value").Value;
            if (xel.Element(ns + "UniqueKeyColumns") != null)
                result.UniqueKeyColumns = xel.Element(ns + "UniqueKeyColumns").Attribute("value").Value;
            if (xel.Element(ns + "ExcludeFromCompareColumns") != null)
                result.ExcludeFromCompareColumns = xel.Element(ns + "ExcludeFromCompareColumns").Attribute("value").Value;

            return result;
        }
    }
}
