﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.IO;

using DWJobs.Common;
using DWJobs.TabularSetsComparer.InputConfigXml;

using log4net;

namespace DWJobs.TabularSetsComparer
{
    public class MdxTabularSetsComparer : TabularSetsComparer
    {
        public MdxTabularSetsComparer(CompareScenario compareScenario, CompareDatabase compareDB, ILog log)
            : base(compareScenario, compareDB, log)
        {
        }

        public override bool CompareResultSets (out string compareResultCSVFileNames)
        {
            return CompareResultSetsForEffectiveDate(null, out compareResultCSVFileNames );
        }

        protected override bool CompareResultSetsForEffectiveDate(DateTime? effDate, out string compareResultCSVFileName)
        {
            compareResultCSVFileName = FileHelper.GetFilePathForTimestampFile(ScenarioToUseForCompare.CompareResultsFolderPath, ScenarioToUseForCompare.ScenarioName, "csv", true);

            int procReturnValue = CompareDB.CompareTablularResults(compareResultCSVFileName,
                                                                    ScenarioToUseForCompare.SSASConnectionStringLeft,
                                                                    ScenarioToUseForCompare.SSASConnectionStringRight,
                                                                    ScenarioToUseForCompare.AliasLeft,
                                                                    ScenarioToUseForCompare.AliasRight,
                                                                    ScenarioToUseForCompare.QueryLeft,
                                                                    ScenarioToUseForCompare.QueryRight,
                                                                    ScenarioToUseForCompare.ResultSetColumnsAndTypes,
                                                                    ScenarioToUseForCompare.UniqueKeyColumns,
                                                                    ScenarioToUseForCompare.ExcludeFromCompareColumns,
                                                                    ScenarioToUseForCompare.FloatCompareThreshold);
        
        
            bool differencesFound = ( procReturnValue > 0 ? true : false );

            if ( differencesFound )
                compareResultCSVFileName = "\n\tScenario '" 
                                + ScenarioToUseForCompare.ScenarioName 
                                + "' - differences: " + "\"" 
                                + "file://" +  compareResultCSVFileName + "\"";
            else
            {
                File.Delete ( compareResultCSVFileName );
                compareResultCSVFileName = "\n\tScenario '" 
                                + ScenarioToUseForCompare.ScenarioName 
                                + "' - differences not found";
            }
            return differencesFound;
        }
    }
}
