﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;

using DWJobs.TabularSetsComparer.InputConfigXml;

using log4net;

namespace DWJobs.TabularSetsComparer
{
    public abstract class TabularSetsComparer : ITabularSetsComparer
    {
        private CompareScenario compareScenario;
        private CompareDatabase compareDB;
        private ILog log = null;

        protected static string[] DATE_FORMATS = { "yyyyMMdd", "yyyy-MM-dd", "MM/dd/yyyy" };
        protected TabularSetsComparer(CompareScenario compareScenario, CompareDatabase compareDB, ILog log)
        {
            this.compareScenario = compareScenario;
            this.compareDB = compareDB;
            this.log = log;
        }

        protected bool CompareResultSetsForEffectiveDates(List<DateTime?> datesToProcess, out string compareResultCSVFileNames)
        {
            compareResultCSVFileNames = "";
            bool anyDifferencesFound = false;
            if (datesToProcess == null)
            {
                anyDifferencesFound = CompareResultSetsForEffectiveDate(null, out compareResultCSVFileNames);
                compareResultCSVFileNames += "\n\n";
            }
            else
            {
                string compareResultCSVFileName;
                foreach (var effDateToLoad in datesToProcess)
                {
                    bool differencesFound = CompareResultSetsForEffectiveDate(effDateToLoad, out compareResultCSVFileName);
                    if (!anyDifferencesFound)
                        anyDifferencesFound = differencesFound;
                    compareResultCSVFileNames += compareResultCSVFileName + "\n\n";
                }
            }
            return anyDifferencesFound;
        }

        protected CompareScenario ScenarioToUseForCompare
        {
            get
            {
                return this.compareScenario;
            }
        }

        protected CompareDatabase CompareDB
        {
            get
            {
                return this.compareDB;
            }
        }

        protected ILog Log
        {
            get
            {
                return this.log;
            }
        }

        public abstract bool CompareResultSets(out string compareResultCSVFileNames);

        protected abstract bool CompareResultSetsForEffectiveDate(DateTime? effDate, out string compareResultCSVFileName);
    }
}